/**
  ******************************************************************************
  * @file    PT32x031x_adc.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1 
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Analog to Digital Convertor (ADC) peripheral:
  *           + Initialization and Configuration
  *           + ADC Channels Configuration
  *           + Interrupts and flags management
  *
  * @attention
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_adc.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup ADC 
  * @brief ADC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_ADC_CR1_MASK           ((u32)0xF1B3F)   //!< CR1 MASK 

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup ADC_Private_Functions
  * @{
  */

/**
  * @brief ADC Initialization
  * @param ADCx:ADC Peripheral
  * @param ADC_InitStruct:ADC_InitTypeDef type structure
  * @retval None
  */
void ADC_Init(ADC_TypeDef *ADCx, ADC_InitTypeDef *ADC_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Prescaler(ADC_InitStruct->ADC_Prescaler));
	assert_param(IS_ADC_Mode(ADC_InitStruct->ADC_Mode));
	assert_param(IS_ADC_TriggerSource(ADC_InitStruct->ADC_TriggerSource));
	assert_param(IS_ADC_TimerTriggerSource(ADC_InitStruct->ADC_TimerTriggerSource));
	assert_param(IS_ADC_Align(ADC_InitStruct->ADC_Align));
	assert_param(IS_ADC_Channel(ADC_InitStruct->ADC_Channel));

	tmpreg = ADCx->CR1;
	tmpreg &= (~Init_ADC_CR1_MASK);
	tmpreg |= ((ADC_InitStruct->ADC_Mode) | \
				 (ADC_InitStruct->ADC_Prescaler) | \
			   (ADC_InitStruct->ADC_TriggerSource) | \
			   (ADC_InitStruct->ADC_TimerTriggerSource) | \
			   (ADC_InitStruct->ADC_Align) | \
			   (ADC_InitStruct->ADC_Channel));
	ADCx->CR1 = tmpreg;
}

/**
  * @brief ADC Structure Initialization
  * @param ADC_InitStruct:ADC_InitTypeDef type structure
  * @retval None
  */
void ADC_StructInit(ADC_InitTypeDef *ADC_InitStruct)
{
	ADC_InitStruct->ADC_Prescaler = ADC_Prescaler_2;
	ADC_InitStruct->ADC_Mode = ADC_Mode_Single;
	ADC_InitStruct->ADC_TriggerSource = ADC_TriggerSource_Software;
	ADC_InitStruct->ADC_TimerTriggerSource = ADC_TimerTriggerSource_TIM1ADC;
	ADC_InitStruct->ADC_Align = ADC_Align_Right;
	ADC_InitStruct->ADC_Channel = ADC_Channel_0;
}

/**
  * @brief 	ADC enable command
  * @param 	ADCx:ADC Peripheral
	* @param  NewState: new state of the ADC peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_Cmd(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Set the ADEN bit to Enable the ADC peripheral */
		ADCx->CR1 |= ADC_CR1_EN;
	}
	else
	{
		ADCx->CR1 &= (~ADC_CR1_EN);
	}
}

/**
  * @brief  Configure the ADC to be clocked by the synchronous clock derived from
  *         the PCLK clock of the ADC bus interface divided by 1,2,4 or 8.
  * @note   This function can be called only when ADC is disabled.
  * @param  ADCx: select the ADC peripheral.
	          @arg  ADC
  * @param  ADC_PSC: This parameter can be :
  *            @arg ADC_Prescaler_1: ADC clocked by PCLK
  *            @arg ADC_Prescaler_2: ADC clocked by PCLK/2
  *            @arg ADC_Prescaler_4: ADC clocked by PCLK/4  
  *            @arg ADC_Prescaler_8: ADC clocked by PCLK/8  
  * @retval None
  */
void ADC_PrescalerConfig(ADC_TypeDef* ADCx, u32 ADC_PSC)
{
  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  assert_param(IS_ADC_Prescaler(ADC_PSC));

  ADCx->CR1 &=(u32)~ADC_CR1_PSC;
  /* Configure the ADC Clock mode according to ADC_PSC */
  ADCx->CR1 |= (u32)ADC_PSC;

}

/**
  * @brief ADC trigger source config
  * @param ADCx:ADC Peripheral
  * @param SEL: ADC Trigger Source Selection
	*         This parameter can be one of the following values:
	*					@arg ADC_TriggerSource_Software: Software Trigger
	*					@arg ADC_TriggerSource_Timer: Software Trigger
	*					@arg ADC_TriggerSource_ExternalPin: ExternalPin Trigger
  * @retval None
  */
void ADC_TriggerSource(ADC_TypeDef *ADCx, u32 SEL)
{
	/* Check the parameters */
	u32 tmpreg;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_TriggerSource(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= ~((u32)ADC_CR1_TRIGS);
	/* Configure the ADC Clock mode according to ADC_ClockMode */
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}

/**
  * @}
  */


/**
* @brief  Enable the Continuous mode for the selected ADCx channels.
* @param  ADCx: select the ADC peripheral.
@arg  ADC
* @param  NewState: new state of the Continuous mode.
*         This parameter can be: ENABLE or DISABLE.
@arg  ENABLE
@arg  DISABLE
* @note   It is not possible to have both discontinuous mode and continuous mode
*         enabled. In this case (If DISCEN and CONT are Set), the ADC behaves 
*         as if continuous mode was disabled.
* @retval None
*/
void ADC_Mode(ADC_TypeDef* ADCx, u32 SEL)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Mode(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= (~ADC_CR1_MODE);
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}

/**
  * @brief 	ADC Channel Config
  * @param 	ADCx:ADC Peripheral
  * @param 	ADC_Channel:the ADC channel to configure.
  *    @arg   This parameter can be ADC_Channel_x where x can be(0..12)
  * @retval None
  */
void ADC_ChannelConfig(ADC_TypeDef *ADCx, u32 ADC_Channel)
{

	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Channel(ADC_Channel));
	/* Clear the ADC Channel Selection bits */
	ADCx->CR1 &= (u32)~ADC_CR1_CHS;
	/* Configure the ADC Channel */
	ADCx->CR1 |= (u32)ADC_Channel;
}

/**
  * @brief ADC Start Of Conversion
  * @param ADCx:ADC Peripheral
  * @retval None
  */
void ADC_StartOfConversion(ADC_TypeDef* ADCx)
{
  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  ADCx->CR1 |= (u32)ADC_CR1_SOC;
}

/**
  * @brief  Returns the last ADCx conversion result data for ADC channel.  
  * @param  ADCx: select the ADC peripheral.
	          @arg  ADC
  * @retval The Data conversion value.
  */
u16 ADC_GetConversionValue(ADC_TypeDef* ADCx)
{
  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));

  /* Return the selected ADC conversion value */
  return (u16) ADCx->DR;
}

/**
  * @brief  Enables or disables the specified ADC interrupts.
  * @param  ADCx: where x can be 1 to select the ADC peripheral.
  * @param  ADC_IT: specifies the ADC interrupt sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *            @arg ADC_IT_EOC: End of conversion interrupt 
  * @param  NewState: new state of the specified ADC interrupts.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_ITConfig(ADC_TypeDef* ADCx, u32 ADC_IT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_ADC_IT(ADC_IT)); 

  if (NewState != DISABLE)
  {
    /* Enable the selected ADC interrupts */
    ADCx->CR1 |= ADC_IT;
  }
  else
  {
    /* Disable the selected ADC interrupts */
    ADCx->CR1 &= (~(u32)ADC_IT);
  }
}

/**
  * @brief ADC Get Flag Status
  * @param ADCx:ADC Peripheral
  * @param ADC_FLAG: specifies the flag to check.
	*          This parameter can be one of the following values:
	*		 				@arg ADC_FLAG_RDY: ADC ready flag
	*		 				@arg ADC_FLAG_EOC: ADC end of conversion flag
  * @retval FlagStatus
  */
FlagStatus ADC_GetFlagStatus(ADC_TypeDef* ADCx, u32 ADC_FLAG)
{
  FlagStatus bitstatus = RESET;

  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  assert_param(IS_ADC_GET_FLAG(ADC_FLAG));
  /* Check the status of the specified ADC flag */
  if ((ADCx->SR & ADC_FLAG) != (u32)RESET)
  {
    /* ADC_FLAG is set */
    bitstatus = SET;
  }
  else
  {
    /* ADC_FLAG is reset */
    bitstatus = RESET;
  }
  /* Return the ADC_FLAG status */
  return  bitstatus;
}

/**
  * @brief  Clears the ADCx's pending flags.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral.
  * @param  ADC_FLAG: specifies the flag to clear. 
  *          This parameter can be any combination of the following values:
  *            @arg ADC_FLAG_EOC: End of conversion flag
  * @retval None
  */
void ADC_ClearFlag(ADC_TypeDef* ADCx, u32 ADC_FLAG)
{
  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  assert_param(IS_ADC_CLEAR_FLAG(ADC_FLAG));

  /* Clear the selected ADC flags */
  ADCx->SR = (u32)ADC_FLAG;
}

/**
  * @brief  Checks whether the specified ADC interrupt has occurred or not.
  * @param  ADCx: where x can be 1 to select the ADC1 peripheral
  * @param  ADC_IT: specifies the ADC interrupt source to check.
  *          This parameter can be one of the following values:
  *            @arg ADC_IT_EOC: End of conversion interrupt 
  * @retval The new state of ADC_IT (SET or RESET).
  */
ITStatus ADC_GetITStatus(ADC_TypeDef* ADCx, u32 ADC_IT)
{
  ITStatus bitstatus = RESET;
  u32 enablestatus = 0;

  /* Check the parameters */
  assert_param(IS_ADC_ALL_PERIPH(ADCx));
  assert_param(IS_ADC_GET_IT(ADC_IT));

  /* Get the ADC_IT enable bit status */
  enablestatus = (u32)(ADCx->CR1 & ADC_IT); 

  /* Check the status of the specified ADC interrupt */
  if (((u32)(ADCx->SR & ADC_IT) != (u32)RESET) && (enablestatus != (u32)RESET))
  {
    /* ADC_IT is set */
    bitstatus = SET;
  }
  else
  {
    /* ADC_IT is reset */
    bitstatus = RESET;
  }
  /* Return the ADC_IT status */
  return  bitstatus;
}

/**
  * @brief ADC Ready Time Config
  * @param ADCx:ADC Peripheral
	* @param ReadyTime:set ready time for the specified ADC
	*          This parameter can be a number between 0x00 and 0xFF.
  * @retval None
  */
void ADC_ReadyTimeConfig(ADC_TypeDef *ADCx, u32 ReadyTime)
{
	u32 tmpreg = 0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_ReadyTime(ReadyTime));

	tmpreg = ADCx->CR2;
	tmpreg &= (~ADC_CR2_RDTC);
	tmpreg |= ReadyTime;
	ADCx->CR2 = tmpreg;
}

/**
  * @}
  */ 

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
