/**
  ******************************************************************************
  * @file    PT32x031x_beep.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1  
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Buzzer driver peripheral:
  *           + Initialization and Configuration
  *           + 
  *           + Interrupts and flags management
  *
  *
  ******************************************************************************
  * @attention
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_beep.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup Buzzer 
  * @brief Buzzer driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/** @defgroup Buzzer_Group1 Configuration of the Buzzer computation unit functions
 *  @brief   Configuration of the Buzzer  unit functions 
 *
@verbatim
 ===============================================================================
                     ##### Buzzer configuration functions #####
 ===============================================================================

@endverbatim
  * @{
  */
/** @defgroup Buzzer_Private_Functions
  * @{
  */
/**
  * @brief  Initializes the beep peripheral according to 
  *         the specified parameters in the Beep_InitStruct.
  * @param  Beep_InitStruct:Buzzer initialization structure.
            @param  Subdivide
						        @arg  BEEP_Divider_X2
                    @arg  BEEP_Divider_X4
                    @arg	BEEP_Divider_X8									
						@param  Prescaler	       
  * @retval None
  */	
void Beep_Init(Beep_InitTypeDef* Beep_InitStruct)
{
  u32 tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_BEEP_SUBDIV(Beep_InitStruct->Subdivide));
  assert_param(IS_BEEP_CPSR(Beep_InitStruct->Prescaler));
  /*  */
  tmpreg = BEEP->CR;
  /*  */
  tmpreg &= (u32)~(u32)(BEEP_CR_PSC | BEEP_CR_DIV);
  /*  */
  tmpreg  |= ((u32)(Beep_InitStruct->Subdivide));
	tmpreg  |= ((u32)((Beep_InitStruct->Prescaler)<<0x01));

  BEEP->CR = tmpreg;	
}

/**
  * @brief  Enables or disables the  BEEP peripheral.
  * @param  NewState: new state of the USARTx peripheral.
  *         This parameter can be: ENABLE or DISABLE.
            @arg   ENABLE
            @arg   DISABLE
  * @retval None
  */
void Beep_Cmd(FunctionalState NewState)
{ 
	/* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if (NewState != DISABLE)
  {
    BEEP->CR |= BEEP_CR_EN;
  }
  else
  {
    BEEP->CR &= (u32)~((u32)BEEP_CR_EN);
  }
}


/**
  * @brief  Set the clock pre-division factor of buzzer
  * @param  Prescaler:Clock predivision factor.
  * @retval 
  */
void Beep_PrescalerConfig(uint8_t Prescaler)
{
  u32 tmpreg = 0;
  /* Check the parameters */
  assert_param(IS_BEEP_CPSR(Prescaler));
  /*  */
  tmpreg = BEEP->CR;
  /*  */
  tmpreg &= (u32)~(u32)(BEEP_CR_PSC);
  /*  */
	tmpreg  |= ((u32)(Prescaler<<0x01));
  /* Write to CRC CR */
  BEEP->CR = tmpreg;	
}

/**
  * @brief  Select the output frequency of buzzer 
  * @param  Subdivide: output frequency
						 @arg  BEEP_Divider_X2
             @arg  BEEP_Divider_X4
             @arg	 BEEP_Divider_X8	            
  * @param 
  * @retval 
  */
void Beep_DividerConfig(uint8_t Subdivide)
{
  u32 tmpreg = 0;
  /* Check the parameters */
  assert_param(IS_BEEP_SUBDIV(Subdivide));
  tmpreg = BEEP->CR;
  tmpreg &= (u32)~(u32)(BEEP_CR_DIV);
	tmpreg  |= ((u32)(Subdivide));
  BEEP->CR = tmpreg;	
}

/**
  * @brief  Enables or disables the  BEEP High Sink .
	* @param  BEEPx: select the BEEPx peripheral.
  * @param  NewState: new state of the BEEP peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void Beep_HighSinkConfig(BEEP_TypeDef *BEEPx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_BEEP_ALL_PERIPH(BEEPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/**/
		BEEPx->HCR |= BEEP_HCR_BEEPE;
	}
	else
	{
		/* */
		BEEPx->HCR &= ~((u32)BEEP_HCR_BEEPE);
	}
}
/**
  * @}
  */

/**
  * @}
  */
   
  /**
  * @}
  */ 

/**
  * @}
  */ 

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
