/**
  ******************************************************************************
  * @file    PT32x031x_cmp.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1 
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Analog comparators and operational amplifiers peripheral:
  *           + Initialization and Configuration
  *           + 
  *           + Interrupts and flags management
  *
  ******************************************************************************
  * @attention
  *
  * 
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_cmp.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup Comparators
  * @brief CMP driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CMP_CR1_MASK 		((u32)0x00001FD)
#define CMP_Trim_KEY  (u32)((u32)0xACDE<<16)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup CMP_Private_Functions
  * @{
  */

/** @defgroup CMP_Group1 Initialization and Configuration functions
 *  @brief   Initialization and Configuration functions 
 *
@verbatim
 ===============================================================================
          ##### Initialization and Configuration functions #####
 ===============================================================================
    [..] This section provides functions allowing to:

   
@endverbatim
  * @{
  */

/**
  * @brief  Initializes the CMP peripheral according to the specified parameters
  *         in the CMP_InitStruct.
  * @note   This function is used to configure the global features of the CMP.
  * @param  CMPx: where x can be 0,1 to select the CMP peripheral.
	          @arg  CMP0
						@arg  CMP1							
  * @retval None
  */
void CMP_Init(CMP_TypeDef* CMPx, CMP_InitTypeDef* CMP_InitStruct)
{
  u32 tmpreg1 = 0;
	u32 tmpreg2 = 0;
  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_CMP_PositiveInput(CMP_InitStruct->CMP_PositiveInput));
	assert_param(IS_CMP_DigitalFilter(CMP_InitStruct->CMP_DigitalFilter));
  assert_param(IS_CMP_PowerMode(CMP_InitStruct->CMP_PowerMode));
	assert_param(IS_CMP_OutputPolarity(CMP_InitStruct->CMP_OutputPolarity));
	assert_param(IS_CMP_InitializationDelayTime(CMP_InitStruct->CMP_InitializationDelayTime));
	
  
	tmpreg1 = CMPx->CR1;
  tmpreg1 &= (u32)~Init_CMP_CR1_MASK;
 	tmpreg1  |= ((u32)((CMP_InitStruct->CMP_PositiveInput) | \
					  (CMP_InitStruct->CMP_DigitalFilter) | \
					  (CMP_InitStruct->CMP_PowerMode)			| \
					  (CMP_InitStruct->CMP_OutputPolarity)));
  CMPx->CR1 = tmpreg1;
	
	tmpreg2 = CMPx->CR3;
	tmpreg2 &=((u32)~CMP_CR3_DELAY);
	tmpreg2 |= ((u32)(CMP_InitStruct->CMP_InitializationDelayTime));
	CMPx->CR3 = tmpreg2;
}


/**
  * @brief  Enables or disables the specified CMP peripheral.
  * @param  CMPx: where x can be 1 to select the CMPx peripheral.
	          @arg  CMP0
						@arg  CMP1
  * @param  NewState: new state of the CMPx peripheral. 
  *         @arg ENABLE 
            @arg DISABLE
  * @retval None
  */
void CMP_Cmd(CMP_TypeDef* CMPx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    CMPx->CR1 |= (u32)CMP_CR1_EN;
  }
  else
  {
    CMPx->CR1 &= (u32)~((u32)CMP_CR1_EN);
  }
}

/**
  * @brief  Enables or disables the OPA mode.
  * @param  CMPx: where x can be 1 to select the CMPx peripheral.
	          @arg  CMP0
						@arg  CMP1
  * @param  NewState: new state of the CMPx peripheral. 
  *         @arg ENABLE 
            @arg DISABLE
  * @retval None
  */
void CMP_ModeConfig(CMP_TypeDef* CMPx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    CMPx->CR2 &= (u32)~((u32)CMP_CR2_MODE);
  }
  else
  {
    CMPx->CR2 |= (u32)CMP_CR2_MODE;
  }
}

/**
  * @brief  Configure CMP initialization delay.
  * @param  CMP: where x can be 1 to select the CMP peripheral.
            @arg CMP0
            @arg CMP1
  * @param  CMP_InitaDelay: Initialization delay time configuration.  ( 0~0x3FFF)    
            This parameter is between 0 and 0~0x3FFF.
  * @retval None
  */
void CMP_SetInitDelayValue(CMP_TypeDef* CMPx, u32 CMP_InitaDelay)
{
  /* Check the parameters */
	u32 tmpreg=0;
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_InitializationDelayTime(CMP_InitaDelay));
	tmpreg = CMPx->CR3;
	tmpreg &=((u32)~CMP_CR3_DELAY);
	tmpreg |= CMP_InitaDelay;
	CMPx->CR3 = tmpreg;

}

/**
  * @brief  Enables or disables the specified CMP interrupts.
  * @param  CMP: where x can be 1 to select the CMP peripheral.
            @arg  CMP0
            @arg  CMP1
  * @retval None
  */
void CMP_ITConfig(CMP_TypeDef* CMPx, u32 CMP_IT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_CMP_IT(CMP_IT)); 

  if (NewState != DISABLE)
  {
    /* Enable the selected CMP interrupts */
    CMPx->IE |= CMP_IT;
  }
  else
  {
    /* Disable the selected CMP interrupts */
    CMPx->IE &= (~(u32)CMP_IT);
  }
}

/**
  * @brief  Checks whether the specified CMP flag is set or not.
  * @param  CMPx: where x can be 0,1 to select the CMP peripheral.
            @arg CMP0
            @arg CMP1
  * @param  CMP_FLAG: specifies the flag to check. 
  *         This parameter can be one of the following values:
  *         @arg CMP_FLAG_FALL
  *         @arg CMP_FLAG_RISE
            @arg CMP_FLAG_FALL_WEAK
            @arg CMP_FLAG_RISE_WEAK
  * @retval The new state of CMP_FLAG (SET or RESET).
  */
FlagStatus CMP_GetFlagStatus(CMP_TypeDef* CMPx, u32 CMP_FLAG)
{
  FlagStatus bitstatus = RESET;

  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_CMP_GET_FLAG(CMP_FLAG));
  /* Check the status of the specified ADC flag */
  if ((CMPx->SR & CMP_FLAG) != (u32)RESET)
  {
    /* CMP_FLAG is set */
    bitstatus = SET;
  }
  else
  {
    /* CMP_FLAG is reset */
    bitstatus = RESET;
  }
  /* Return the CMP_FLAG status */
  return  bitstatus;
}

/**
  * @brief  Clears the CMP's pending flags.
  * @param  CMPx: where x can be 0,1 to select the CMP peripheral.
            @arg CMP0
            @arg CMP1
  * @param  CMP_FLAG: specifies the flag to clear.  
            @arg CMP_FLAG_FALL
            @arg CMP_FLAG_RISE
            @arg CMP_FLAG_FALL_WEAK
            @arg CMP_FLAG_RISE_WEAK 
  * @retval None
  */
void CMP_ClearFlag(CMP_TypeDef* CMPx, u32 CMP_FLAG)
{
  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_CMP_CLEAR_FLAG(CMP_FLAG));

  /* Clear the selected CMP flags */
  CMPx->SR = (u32)CMP_FLAG;
}
void CMP_TrimMode(CMP_TypeDef* CMPx,FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the selected CMP interrupts */
    CMPx->CR2 |= CMP_CR2_TME;
  }
  else
  {
    /* Disable the selected CMP interrupts */
    CMPx->CR2 &= (~(u32)CMP_CR2_TME);
  }	
}

void CMP_TrimValue(CMP_TypeDef* CMPx,u32 value)
{
  u32 tmpreg=0;
	/* Check the parameters */
  assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_TrimValue(value));
	
	tmpreg=CMPx->CR2;
	tmpreg &=(u32)(~CMP_CR2_TRIM);
	tmpreg |= (u32)((value<<4)|CMP_Trim_KEY);
	CMPx->CR2=tmpreg;
}

/**
  * @}
  */


/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
