/********************************************************************************
  * @file    PT32x031_crc.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/12/1
  * @brief   This file contains all the functions prototypes for the CRC firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_crc.h"

/** @defgroup CRC
  * @brief CRC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CRC_CR_MASK		((u32)0xFFFFFFC0)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief Initializes the CRC peripheral according to the specified parameters
  *         in the CRC_InitStruct.
  * @param CRC_InitStruct:pointer to an CRC_InitTypeDef structure that contains
  *         the configuration information for the specified CRC peripheral.
  * @retval None
  */
void CRC_Init(CRC_InitTypeDef *CRC_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_CRC_Input(CRC_InitStruct->CRC_Input));
	assert_param(IS_CRC_InputBitSequenceReversal(CRC_InitStruct->CRC_InputBitSequenceReversal));
	assert_param(IS_CRC_InputByteSequenceReversal(CRC_InitStruct->CRC_InputByteSequenceReversal));
	assert_param(IS_CRC_OutputBitSequenceReversal(CRC_InitStruct->CRC_OutputBitSequenceReversal));
	assert_param(IS_CRC_Seed(CRC_InitStruct->CRC_Seed));
	assert_param(IS_CRC_Poly(CRC_InitStruct->CRC_Poly));
	/*  */
	tmpreg = CRC->CR;
	/*  */
	tmpreg &= Init_CRC_CR_MASK;
	/*  */
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_Input));
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_InputBitSequenceReversal));
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_InputByteSequenceReversal));
	tmpreg  |= ((u32)(CRC_InitStruct->CRC_OutputBitSequenceReversal));
	/* Write to CRC CR */
	CRC->CR = tmpreg;
	CRC->SEED = CRC_InitStruct->CRC_Seed;
	CRC->POLY = CRC_InitStruct->CRC_Poly;
}

/**
  * @brief  Fills each CRC_InitStruct member with its default value.
  * @param  CRC_InitStruct: pointer to an CRC_InitTypeDef structure which will be initialized.
  * @retval None
  */
void CRC_StructInit(CRC_InitTypeDef *CRC_InitStruct)
{
	/* Initialize the CRC data input width */
	CRC_InitStruct->CRC_Input = CRC_Input_16b;
	/* Initialize the CRC data input bit reverse function */
	CRC_InitStruct->CRC_InputBitSequenceReversal = CRC_InputBitSequenceReversal_Disable;
	/* Initialize the CRC data output bit reverse function */
	CRC_InitStruct->CRC_OutputBitSequenceReversal = CRC_OutputBitSequenceReversal_Disable;
	/* Initialize the CRC data input byte reverse function */
	CRC_InitStruct->CRC_InputByteSequenceReversal = CRC_InputByteSequenceReversal_Enable;
	/* Initialize the CRC POLY */
	CRC_InitStruct->CRC_Poly = CRC_CCITT16;
	/* Initialize the CRC SEED */
	CRC_InitStruct->CRC_Seed = 0;
}

/**
	* @brief  Enables or disables the specified CRC peripheral.
  * @param  NewState: new state of the CRC peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CRC_Cmd(FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Enable the selected CRC peripheral */
		CRC->CR |= CRC_CR_EN;
	}
	else
	{
		/* Disable the selected CRC peripheral */
		CRC->CR &= (u16)~((u16)CRC_CR_EN);
	}
}

/**
  * @brief Reset data poutput of the specified CRC peripheral.
  * @retval None
  */
void CRC_ResetDout(void)
{
	CRC->CR |= CRC_CR_CRS;
}

/**
  * @brief CRC Calculate CRC
  * @param CRC_Data: the data will be writed into CRC->DIN
  * @retval CRC->DOUT
  */
u16 CRC_CalculateCRC(u16 CRC_Data)
{
	CRC->DIN = (u16) CRC_Data;
	return (CRC->DOUT);
}

/**
  * @brief CRC Calculate Block CRC
  * @param pBuffer: the data will be writed into CRC->DIN
  * @param BufferLength: the length of pBuffer
  * @retval CRC->DOUT
  */
u16 CRC_CalculateBlockCRC(u16 pBuffer[], u32 BufferLength)
{
	u32 index = 0;

	for(index = 0; index < BufferLength; index++)
	{
		CRC->DIN = pBuffer[index];
	}

	return (CRC->DOUT);
}

/**
  * @brief CRC Get data output
  * @retval CRC->DOUT
  */
u16 CRC_GetCRC(void)
{
	return (CRC->DOUT);
}

/**
  * @brief Set CRC seed.
  * @retval None
  */
void CRC_Seed(u16 Seed)
{
	assert_param(IS_CRC_Seed(Seed));
	
	CRC->SEED = Seed;	
}

/**
  * @brief Set CRC poly.
  * @retval None
  */
void CRC_Poly(u16 Poly)
{
	assert_param(IS_CRC_Poly(Poly));
	
	CRC->POLY = Poly;	
}

/**
  * @}
  */
