/**
  ******************************************************************************
  * @file    PT32x031x_i2c.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Inter-Integrated circuit (I2C):
  *           + Initialization and Configuration
  *           + Communications handling
  *           + I2C registers management
  *           + Data transfers management
  *           + Interrupts and flags management
  *
  ******************************************************************************
  * @attention
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_i2c.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup I2C 
  * @brief I2C driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup I2C_Private_Functions
  * @{
  */


/** @defgroup I2C_Group1 Initialization and Configuration functions
 *  @brief   Initialization and Configuration functions 
 *
@verbatim   
 ===============================================================================
           ##### Initialization and Configuration functions #####
 ===============================================================================
    [..] This section provides a set of functions allowing to initialize the I2C Mode,
         I2C Timing, I2C filters, I2C Addressing mode, I2C OwnAddress1.

    [..] The I2C_Init() function follows the I2C configuration procedures (these procedures 
         are available in reference manual).

@endverbatim
  * @{
  */



/**
  * @brief  Initializes the I2Cx peripheral according to the specified
  *         parameters in the I2C_InitStruct.
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg  I2C0
						@arg  I2C1
  * @param  I2C_InitStruct: pointer to a I2C_InitTypeDef structure that
  *         contains the configuration information for the specified I2C peripheral.
	          @param  I2C_OwnAddressSlave address
						        This parameter is between 0 and 0x7F
						@param  I2C_AcknowledgeResponse bit control
  *                 @arg  ENABLE 
  *                 @arg  DISABLE						
						@param  I2C_SCLK_DIVFrequency division coefficient of communication clock
						        @arg  I2C_Prescaler_DIV60
										@arg  I2C_Prescaler_DIV120
										@arg  I2C_Prescaler_DIV160
										@arg  I2C_Prescaler_DIV192
										@arg  I2C_Prescaler_DIV244
										@arg  I2C_Prescaler_DIV960
										@arg  I2C_Prescaler_DIV256
						@param  I2C_BroadcastBroadcast addressing enable bit.
						  *     @arg  ENABLE 
  *                 @arg  DISABLE	
  * @retval None
  */
void I2C_Init(I2C_TypeDef* I2Cx, I2C_InitTypeDef* I2C_InitStruct)
{
  u32 tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_I2C_ALL_PERIPH(I2Cx));
  assert_param(IS_I2C_AddressValue(I2C_InitStruct->I2C_OwnAddress));
  assert_param(IS_I2C_Acknowledge(I2C_InitStruct->I2C_Acknowledge));
  assert_param(IS_I2C_Prescaler_DIV(I2C_InitStruct->I2C_Prescaler));
  assert_param(IS_I2C_Broadcast(I2C_InitStruct->I2C_Broadcast));

  /* Disable I2Cx Peripheral */
  I2Cx->CCR = I2C_CCR_MASK;

  /*---------------------------- I2Cx BroadCast Configuration ---------------------*/
  /*---------------------------- I2Cx OAR Configuration ---------------------*/
	tmpreg = (u32)(((I2C_InitStruct->I2C_OwnAddress) << 0x01) | (I2C_InitStruct->I2C_Broadcast));
	/* Write to I2Cx OAR */
	I2Cx->OAR = tmpreg;
  /*---------------------------- I2Cx ACK Configuration ----------------------*/
	I2Cx->CR = I2C_Ack_Enable;
  /*---------------------------- I2Cx Frequency division factor Configuration  ----------------------*/
  I2Cx->CR	= I2C_InitStruct->I2C_Prescaler;
	
}

/**
  * @brief  Fills each I2C_InitStruct member with its default value.
  * @param  I2C_InitStruct: pointer to an I2C_InitTypeDef structure which will be initialized.
  */
void I2C_StructInit(I2C_InitTypeDef* I2C_InitStruct)
{
  /*---------------- Reset I2C init structure parameters values --------------*/
  /* Initialize the I2C_OwnAddress1 member */
  I2C_InitStruct->I2C_OwnAddress = 0;
  /* Initialize the I2C_Acknowledge member */
  I2C_InitStruct->I2C_Acknowledge = DISABLE;
  /* Initialize the  Frequency division factor when pclk=24000000 member SCLK=100000 */
  I2C_InitStruct->I2C_Prescaler= I2C_Prescaler_DIV192;
	I2C_InitStruct->I2C_Broadcast=DISABLE;
}

/**
  * @brief  Enables or disables the specified I2C peripheral.
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg I2C0
						@arg I2C1
  * @param  NewState: new state of the I2Cx peripheral. 
  *          This parameter can be: ENABLE or DISABLE.
  *          @arg  ENABLE 
  *          @arg  DISABLE	
  * @retval None
  */
void I2C_Cmd(I2C_TypeDef* I2Cx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_I2C_ALL_PERIPH(I2Cx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    /* Enable the selected I2C peripheral */
    I2Cx->CR = I2C_CR_EN;
  }
  else
  {
    /* Disable the selected I2C peripheral */
    I2Cx->CCR = I2C_CCR_EN;
  }
}

/**
  * @brief  Enables or disables the I2C general call mode.
   * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg I2C0
						@arg I2C1
  * @param  NewState: new state of the I2C general call mode.
  *          This parameter can be: ENABLE or DISABLE.  
  *          @arg  ENABLE 
  *          @arg  DISABLE	
  * @retval None
  */
void I2C_GeneralCallCmd(I2C_TypeDef* I2Cx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_I2C_ALL_PERIPH(I2Cx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if (NewState != DISABLE)
  {
    /* Enable general call mode */
    I2Cx->OAR |= I2C_OAR_BC;
  }
  else
  {
    /* Disable general call mode */
    I2Cx->OAR &= (u32)~((u32)I2C_OAR_BC);
  }
} 


/**
  * @brief  Generates I2Cx communication event.
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
  * @param  Event: the types of I2Cx's event.
	*          This parameter can be one of the following values:
	*          @arg I2C_Event_Stop: I2C generate stop
	*          @arg I2C_Event_Start: I2C generates start
	*          @arg I2C_Event_Restart: I2C generates restart
  * @param  NewState: new state of the I2C Event generation.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void I2C_GenerateEvent(I2C_TypeDef *I2Cx, u8 Event, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_I2C_ALL_PERIPH(I2Cx));
	assert_param(IS_I2C_Event(Event));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	/* Generate a STOP event */
	if(Event == I2C_Event_Stop)
	{
		if(NewState != DISABLE)
		{
			/* Generate a STOP condition */
			I2Cx->CR = I2C_CR_STOP;
			I2Cx->CCR = I2C_CCR_SI;
		}
		else
		{
			/* Disable the STOP condition generation */
			I2Cx->CCR = I2C_CCR_SI | I2C_CCR_STOP;
		}
	}
	/* Generate a START event */
	else if(Event == I2C_Event_Start)
	{
		if(NewState != DISABLE)
		{
			/* Generate a START condition */
			I2Cx->CR = I2C_CR_STAR;
		}
		else
		{
			/* Disable the START condition generation */
			I2Cx->CCR = I2C_CCR_STAR;
		}
	}
	/* Generate a ReSTART event */
	else
	{
		if(NewState != DISABLE)
		{
			/* Generate a START condition */
			I2Cx->CR = I2C_CR_STAR;
			I2Cx->CCR = I2C_CCR_SI;
		}
		else
		{
			/* Disable the START condition generation */
			I2Cx->CCR = I2C_CCR_STAR;
		}
	}
}

/**
  * @brief  Generates I2C communication Acknowledge.
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg I2C0
						@arg I2C1
  * @param  NewState: new state of the Acknowledge.
  *         This parameter can be: ENABLE or DISABLE.  
  *          @arg  ENABLE 
  *          @arg  DISABLE	
  * @retval None
  */
void I2C_AcknowledgeConfig(I2C_TypeDef* I2Cx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_I2C_ALL_PERIPH(I2Cx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if (NewState != DISABLE)
  {
    /* Enable ACK generation */
		I2Cx->CR = I2C_CR_ACK;
  }
  else
  {
    /* Enable NACK generation */
    I2Cx->CCR = I2C_CCR_ACK;
  }
}


/**
  * @brief  Sends a data byte through the I2Cx peripheral.
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg I2C0
						@arg I2C1
  * @param  Data: Byte to be transmitted..
  * @retval None
  */
void I2C_SendData(I2C_TypeDef* I2Cx, u8 Data)
{
  /* Check the parameters */
  assert_param(IS_I2C_ALL_PERIPH(I2Cx));
  
  /* Write in the DR register the data to be sent */
  I2Cx->DR = (u8)Data;
	I2Cx->CCR = I2C_CCR_SI | I2C_CCR_STAR; 
		
}
/**
  * @brief  Sends a data/addr byte through the I2Cx peripheral
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
  * @param  Data: Byte to be transmitted..
  * @retval None
  */
void I2C_SendAddress(I2C_TypeDef *I2Cx, u8 Data)
{
	/* Check the parameters */
	assert_param(IS_I2C_ALL_PERIPH(I2Cx));
	/* Write in the DR register the data to be sent */
	I2Cx->DR = (u8)Data;
	I2Cx->CCR = I2C_CCR_SI | I2C_CCR_STAR;
}
/**
  * @brief  Returns the most recent received data by the I2Cx peripheral.
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg I2C0
						@arg I2C1
  * @retval The value of the received data.
  */
u8 I2C_ReceiveData(I2C_TypeDef* I2Cx)
{
  /* Check the parameters */
  assert_param(IS_I2C_ALL_PERIPH(I2Cx));
  
  /* Return the data in the DR register */
  return (u8)I2Cx->DR;
}  

/**
  * @brief  Return the SR register value 
  * @param  I2Cx: where x can be 0 or 1 to select the I2C peripheral.
	          @arg I2C0
						@arg I2C1
  * @retval The value of the SR register.
  */
FlagStatus I2C_GetFlagStatus(I2C_TypeDef* I2Cx, u16 I2C_FLAG)
{ 
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_I2C_ALL_PERIPH(I2Cx));
	assert_param(IS_I2C_StatusFlag(I2C_FLAG));

	if(I2Cx->SR  == I2C_FLAG)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}

	return bitstatus;
} 


/**
  * @}
  */

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
