/**
  ******************************************************************************
  * @file    PT32x031x_ifmc.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the FLASH peripheral:
  *            - FLASH Interface configuration
  *            - FLASH Memory Programming
  *            - Interrupts and flags management
  *
  ******************************************************************************
  * @attention
  * 
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_ifmc.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup FLASH 
  * @brief FLASH driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
 
/** @defgroup IFMC_Private_Functions
  * @{
  */ 

/** @defgroup IFMC_Group1 FLASH Interface configuration functions
  *  @brief   FLASH Interface configuration functions 
 *
@verbatim   
 ===============================================================================
               ##### FLASH Interface configuration functions #####
 ===============================================================================

    [..] IFMC_Interface configuration_Functions, includes the following functions:
       (+) void IFMC_SetLatency(uint32_t IFMC_WAIT):
    [..] To correctly read data from Flash memory, the number of wait states (LATENCY) 
     must be correctly programmed according to the frequency of the CPU clock (HCLK) 
    [..]
        +--------------------------------------------- +
        |  Wait states  |   HCLK clock frequency (MHz) |
        |---------------|------------------------------|
        |0WS(1CPU cycle)|       0 < HCLK <= 24         |
        |---------------|------------------------------|
        |1WS(2CPU cycle)|      24 < HCLK <= 48        |
        +----------------------------------------------+
    [..]
       (+) void IFMC_PrefetchBufferCmd(FunctionalState NewState);
    [..]
     All these functions don't need the unlock sequence.

@endverbatim
  * @{
  */

/**
  * @brief  Sets the code latency value.
  * @param  IFMC_WAIT: specifies the FLASH Latency value.
  *          This parameter can be one of the following values:
  *             @arg IFMC_Latency_0: FLASH Zero Latency cycle
  *             @arg IFMC_Latency_1: FLASH One Latency cycle
  * @retval None
  */
void IFMC_SetLatency(uint32_t IFMC_WAIT)
{
   uint32_t tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_IFMC_LATENCY(IFMC_WAIT));

  /* Read the CR register */
  tmpreg = IFMC->CR;

  /* Sets the Latency value */
  tmpreg &= (uint32_t) (~((uint32_t)IFMC_CR_WAIT));
  tmpreg |= IFMC_WAIT;

  /* Write the CR register */
  IFMC->CR = ( IFMC_CR_KEY1 | tmpreg );
}

/**
  * @}
  */

/** @defgroup IFMC_Group2 FLASH Memory Programming functions
 *  @brief   FLASH Memory Programming functions
 *
@verbatim   
 ===============================================================================
                ##### FLASH Memory Programming functions #####
 ===============================================================================

    [..] The FLASH Memory Programming functions, includes the following functions:
       (+) IFMC_Status IFMC_ErasePage(uint32_t Page_Address);
       (+) IFMC_Status IFMC_EraseAllPages(void);
       (+) IFMC_Status IFMC_ProgramWord(uint32_t Address, uint32_t Data);

    [..] Any operation of erase or program should follow these steps:
       (#) Call the desired function to erase page or program data

@endverbatim
  * @{
  */
	
/**
  * @brief  Erases a specified page in program memory.
  * @param  Page_Address: The page address in program memory to be erased.
  * @note   A Page is erased in the Program memory only if the address to load 
  *         is the start address of a page (multiple of 512 bytes).
  * @retval FLASH Status: The returned value can be: 
  *         IFMC_ERROR_PROGRAM, IFMC_ERROR_WRP, IFMC_COMPLETE or IFMC_TIMEOUT.
  */
IFMC_Status IFMC_ErasePage(uint32_t Page_Address)
{
  uint32_t tmpreg = 0;		
  IFMC_Status status = IFMC_COMPLETE;

  /* Check the parameters */
  assert_param(IS_IFMC_PROGRAM_ADDRESS(Page_Address));
 
  /* Wait for last operation to be completed */
  status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
  tmpreg = IFMC->CR;

  tmpreg &= ~ (IFMC_OPMOD_WR | IFMC_CR_CMD | IFMC_OPSTR_STRT);	
	
	tmpreg |= (IFMC_OPMOD_WR|IFMC_OPCMD_PERA|IFMC_OPSTR_STRT);	
  
  if(status == IFMC_COMPLETE)
  { 
    IFMC->AR  = Page_Address;
		if(((Page_Address) <= MAINCODE_END_ADDR))
		{
      /* The main code area is erased by page */
		  IFMC->CR = IFMC_CR_KEY1|tmpreg;
      status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);  			
		}
		else
		{		
      /* User information area is erased by page */
      if( ((Page_Address) >= USERINF_START_ADDR) && ((Page_Address) <= USERINF_END_ADDR))
			{				
		    IFMC->CR = IFMC_CR_KEY2|tmpreg;	
			  status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT); 
			}	
		  else
		  {
        status = IFMC_ADDR_ERR; 		
		  }      			
		}
  }
    
  /* Return the Erase Status */
  return status;
}

/**
  * @brief  Erases all FLASH pages.
  * @param  None
  * @retval FLASH Status: The returned value can be: IFMC_ERROR_PG,
  *         IFMC_ERROR_WRP, IFMC_COMPLETE or IFMC_TIMEOUT.
  */
IFMC_Status IFMC_EraseAllPages(void)
{
  IFMC_Status status = IFMC_COMPLETE;
	uint32_t i,tmpreg;
  for(i=0;i<IFMC_DEV_SIZE;i++)
	{
    /* Wait for last operation to be completed */
    status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT); 
		tmpreg = IFMC->CR;
    tmpreg &= ~ (IFMC_OPMOD_WR | IFMC_CR_CMD | IFMC_OPSTR_STRT);	
	  tmpreg |= (IFMC_OPMOD_WR|IFMC_OPCMD_PERA|IFMC_OPSTR_STRT);	
		IFMC->AR  = i*IFMC_PAGE_SIZE;
    if(status == IFMC_COMPLETE)
     {
      /* if the previous operation is completed, proceed to erase all pages */
		  IFMC->CR = IFMC_CR_KEY1|tmpreg;
      /* Wait for last operation to be completed */
      status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
     }
  }

  /* Return the Erase Status */
  return status;
}


/**
  * @brief  Programs a word at a specified address.
  * @param  Address: specifies the address to be programmed.
  * @param  Data: specifies the data to be programmed.
  * @retval FLASH Status: The returned value can be: IFMC_ERROR_PG,
  *         IFMC_ERROR_WRP, IFMC_COMPLETE or IFMC_TIMEOUT. 
  */
IFMC_Status IFMC_ProgramWord(uint32_t Address, uint32_t Data)
{
  IFMC_Status status = IFMC_COMPLETE;
  __IO uint32_t tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_IFMC_PROGRAM_ADDRESS(Address));
  tmpreg = IFMC->CR;
  tmpreg &= ~ (IFMC_OPMOD_WR | IFMC_CR_CMD | IFMC_OPSTR_STRT);		
	tmpreg |= (IFMC_OPMOD_WR|IFMC_OPCMD_PRO|IFMC_OPSTR_STRT);
  /* Wait for last operation to be completed */
  status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
  
  if(status ==IFMC_COMPLETE)
  { 
    /* if the previous operation is completed, proceed to erase all pages */
    IFMC->AR  = Address;
		IFMC->DR  = Data;
		if(((Address) <= MAINCODE_END_ADDR)&&((Address&0x03)==0))
		{
      /* Data is programming into the main code area */
		  IFMC->CR = IFMC_CR_KEY1|tmpreg;
      status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);  			
		}
		else
		{		
      /* Data is programming into the user information area */
      if( ((Address) >= USERINF_START_ADDR) && ((Address) <= USERINF_END_ADDR)&&((Address&0x03)==0))
			{				
		    IFMC->CR = IFMC_CR_KEY2|tmpreg;	
			  status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT); 
			}	
		  else
		  {
        status = IFMC_ADDR_ERR; 		
		  }      			
		}
  }
   
  /* Return the Program Status */
  return status;
}

/**
  * @}
  */

/** @defgroup IFMC_Group2 FLASH Memory Reading functions
 *  @brief   FLASH Memory Reading functions
 *
@verbatim   
 ===============================================================================
                ##### FLASH Memory Reading functions #####
 ===============================================================================

    [..] The FLASH Memory Programming functions, includes the following functions:
       (+) uint32_t IFMC_ReadWord(uint32_t Address);
       (+) uint16_t IFMC_ReadHalfWord(uint32_t Address);
       (+) uint8_t IFMC_ReadByte(uint32_t Address);

@endverbatim
  * @{
  */	
/**
  * @brief  Read a word at a specified address.
  * @note   
  * @note   
  *         
  * @param  
  * @note   
  *         
  * @retval 
  *         
  */
uint32_t IFMC_ReadWord(uint32_t Address)
{
  return *(__IO uint32_t*) Address;
}
/**
  * @brief  Read a half word at a specified address.
  * @note   
  * @note   
  *        
  * @param  
  * @param  
  * @retval 
  *         
  */
uint16_t IFMC_ReadHalfWord(uint32_t Address)
{
  return *(__IO uint16_t*) Address;
}

/**
  * @brief Read a Byte at a specified address. 
  * @note   
  * @note  
  *        
  * @param  
  * @param  
  * @retval 
  *         
  */
uint8_t IFMC_ReadByte(uint32_t Address)
{
  return *(__IO uint8_t*) Address;
}

/**
  * @}
  */ 
/** @defgroup IFMC_Group4 Interrupts and flags management functions
 *  @brief   Interrupts and flags management functions
 *
@verbatim   
 ===============================================================================
             ##### Interrupts and flags management functions #####
 ===============================================================================  

@endverbatim
  * @{
  */
/**
  * @brief  Enables or disables the specified FLASH interrupts.
  * @param  IFMC_IT: specifies the FLASH interrupt sources to be enabled or 
  *         disabled.
  *          This parameter can be any combination of the following values:
  *             @arg IFMC_IT_EOPRO: FLASH end of programming Interrupt
  *             @arg IFMC_IT_EOPRERENDIE: FLASH End of page erase Interrupt
	*             @arg IFMC_IT_CMDER: FLASH command error Interrupt
  *             @arg IFMC_IT_KEYER: FLASH key error Interrupt
	*             @arg IFMC_IT_ADER: FLASH address error Interrupt
  * @param  NewState: new state of the specified FLASH interrupt.
  *          This parameter can be: ENABLE or DISABLE.
	          @arg  ENABLE
            @arg  DISABLE             					
  * @retval None 	
  */
void IFMC_ITConfig(uint32_t IFMC_IT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_IFMC_IT(IFMC_IT)); 
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if(NewState != DISABLE)
  {
    /* Enable the interrupt sources */
    IFMC->IE |= IFMC_IT;
  }
  else
  {
    /* Disable the interrupt sources */
    IFMC->IE &= ~(uint32_t)IFMC_IT;
  }
}

/**
  * @brief  Checks whether the specified FLASH flag is set or not.
  * @param  IFMC_FLAG: specifies the FLASH flag to check.
  *          This parameter can be one of the following values:
  *             @arg IFMC_FLAG_WREND 
  *             @arg IFMC_FLAG_EREND 
  *             @arg IFMC_FLAG_PREREND
  *             @arg IFMC_FLAG_BUSY 
  *             @arg IFMC_FLAG_CMDER  
  *             @arg IFMC_FLAG_KEYER 
  *             @arg IFMC_FLAG_ADER   
  * @retval The new state of IFMC_FLAG (SET or RESET).
  */
FlagStatus IFMC_GetFlagStatus(uint32_t IFMC_FLAG)
{
  FlagStatus bitstatus = RESET;

  /* Check the parameters */
  assert_param(IS_IFMC_GET_FLAG(IFMC_FLAG));

  if((IFMC->SR & IFMC_FLAG) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  /* Return the new state of IFMC_FLAG (SET or RESET) */
  return bitstatus; 
}

/**
  * @brief  Clears the FLASH's pending flags.
  * @param  IFMC_FLAG: specifies the FLASH flags to clear.
  *          This parameter can be any combination of the following values:
  *             @arg IFMC_FLAG_WREND 
  *             @arg IFMC_FLAG_EREND 
  *             @arg IFMC_FLAG_PREREND
  *             @arg IFMC_FLAG_BUSY 
  *             @arg IFMC_FLAG_CMDER  
  *             @arg IFMC_FLAG_KEYER 
  *             @arg IFMC_FLAG_ADER   
  * @retval None
  */
void IFMC_ClearFlag(uint32_t IFMC_FLAG)
{
  /* Check the parameters */
  assert_param(IS_IFMC_CLEAR_FLAG(IFMC_FLAG));
  
  /* Clear the flags */
  IFMC->SR = IFMC_FLAG;
}

/**
  * @brief  Returns the FLASH Status.
  * @param  None
  * @retval FLASH Status: The returned value can be: 
  *         IFMC_BUSY, IFMC_ERROR_PROGRAM, IFMC_ERROR_WRP or IFMC_COMPLETE.
  */
IFMC_Status IFMC_GetStatus(void)
{
  IFMC_Status IFMCstatus = IFMC_COMPLETE;
  

      if((IFMC->SR & (uint32_t)(IFMC_FLAG_ADER)) != (uint32_t)0x00)
      {
        IFMCstatus = IFMC_ADDR_ERR; 
      }
      else
      {
        IFMCstatus = IFMC_COMPLETE;
      }
  /* Return the FLASH Status */
  return IFMCstatus;
}


/**
  * @brief  Waits for a FLASH operation to complete or a TIMEOUT to occur.
  * @param  Timeout: FLASH programming Timeout
  * @retval FLASH Status: The returned value can be: IFMC_BUSY, 
  *         IFMC_ERROR_PROGRAM, IFMC_ERROR_WRP, IFMC_COMPLETE or IFMC_TIMEOUT.
  */
IFMC_Status IFMC_WaitForLastOperation(uint32_t Timeout)
{ 
  IFMC_Status status = IFMC_COMPLETE;
   
  /* Check for the FLASH Status */
  status = IFMC_GetStatus();
  
  /* Wait for a FLASH operation to complete or a TIMEOUT to occur */
  while((status == IFMC_BUSY) && (Timeout != 0x00))
  {
    status = IFMC_GetStatus();
    Timeout--;
  }
  
  if(Timeout == 0x00 )
  {
    status = IFMC_TIMEOUT;
  }
  /* Return the operation status */
  return status;
}

/**
  * @brief  Returns the Remap Status.
  * @param  None
  * @retval Remap Status
  */
ProtectStatus System_GetIFMCProtectStatus(void)
{
  u32 tmp;
  tmp = (IFMC->RPSR & (0x00000001));
  if (tmp != RESET)
  {
    return SET;
  }
  else
  {
    return RESET;
  }
}

/**
  * @}
  */
/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
