/**
  ******************************************************************************
  * @file    PT32x031x_opa.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1 
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Analog comparators and operational amplifiers peripheral:
  *           + Initialization and Configuration
  *           + 
  *           + Interrupts and flags management
  ******************************************************************************
  * @attention
  * 
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_opa.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup Operational Amplifier
  * @brief OPA driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_OPA_CR1_MASK 	((u32)(0x0000008D))
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup OPA_Private_Functions
  * @{
  */

/**
  * @brief  Initializes the OPA peripheral according to the specified parameters
  *         in the OPA_InitStruct.
  * @note   This function is used to configure the global features of the OPA.
  * @param  OPAx: where x can be 0,1 to select the OPA peripheral.
	          @arg  OPA0
						@arg  OPA1							
  * @retval None
  */
void OPA_Init(OPA_TypeDef* OPAx, OPA_InitTypeDef* OPA_InitStruct)
{
  u32 tmpreg1 = 0;
  /* Check the parameters */
  assert_param(IS_OPA_ALL_PERIPH(OPAx));
  assert_param(IS_OPA_PositiveInput(OPA_InitStruct->OPA_PositiveInput));
  assert_param(IS_OPA_PowerMode(OPA_InitStruct->OPA_PowerMode));
	
	tmpreg1 = OPAx->CR1;
  tmpreg1 &= (u32)~Init_OPA_CR1_MASK;
 	tmpreg1  |= ((u32)((OPA_InitStruct->OPA_PositiveInput) | \
					  (OPA_InitStruct->OPA_PowerMode)));
  OPAx->CR1 = tmpreg1;
}

/**
  * @brief  Enables or disables the specified OPAOPA peripheral.
  * @param  OPAx: where x can be 1 to select the OPAx peripheral.
	          @arg  OPA0
						@arg  OPA1
  * @param  NewState: new state of the OPAx peripheral. 
  *         @arg ENABLE 
            @arg DISABLE
  * @retval None
  */
void OPA_Cmd(OPA_TypeDef* OPAx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_OPA_ALL_PERIPH(OPAx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    OPAx->CR1 |= (u32)OPA_CR1_EN;
  }
  else
  {
    OPAx->CR1 &= (u32)~((u32)OPA_CR1_EN);
  }
}

/**
  * @brief  Enables or disables the OPA mode.
  * @param  OPAx: where x can be 1 to select the OPAx peripheral.
	          @arg  OPA0
						@arg  OPA1
  * @param  NewState: new state of the OPAx peripheral. 
  *         @arg ENABLE 
            @arg DISABLE
  * @retval None
  */
void OPA_ModeConfig(OPA_TypeDef* OPAx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_OPA_ALL_PERIPH(OPAx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    OPAx->CR2 |= (u32)OPA_CR2_MODE;
  }
  else
  {
    OPAx->CR2 &= (u32)~((u32)OPA_CR2_MODE);
  }
}

void OPA_TrimMode(OPA_TypeDef* OPAx,FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_OPA_ALL_PERIPH(OPAx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the selected CMP interrupts */
    OPAx->CR2 |= OPA_CR2_TME;
  }
  else
  {
    /* Disable the selected CMP interrupts */
    OPAx->CR2 &= (~(u32)OPA_CR2_TME);
  }	
}

void OPA_TrimValue(OPA_TypeDef* OPAx,u32 value)
{
  u32 tmpreg=0;
	/* Check the parameters */
  assert_param(IS_OPA_ALL_PERIPH(OPAx));
	assert_param(IS_OPA_TrimValue(value));
	
	tmpreg=OPAx->CR2;
	tmpreg &=(u32)(~OPA_CR2_TRIM);
	tmpreg |= (u32)((value<<4)|OPA_Trim_KEY);
	OPAx->CR2=tmpreg;
}


/**
  * @}
  */ 

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
