/**
  ******************************************************************************
  * @file    pt32f0xx_pwm.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1 
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the TIM peripheral:
  *            + TimeBase management
  *            + Output Compare management
  *            + Input Capture management
  *            + Interrupts, flags management
  *            + Clocks management
  *            + Synchronization management     
  *              
  *    
  ******************************************************************************
  * @attention
  * 
  * 
  * 
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_pwm.h"


/** @addtogroup PT32F0xx_StdPeriph_Driver
  * @{
  */

/** @defgroup PWM 
  * @brief PWM driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_OCMR_MASK1(x)	(((u32)0x00000003)<<((x)*4))
#define Init_OCMR_MASK2(x)	(((u32)0x00110000)<<x)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup PWM_Private_Functions
  * @{
  */


/**
  * @brief  Initializes the PWMx Time Base Unit peripheral according to 
  *         the specified parameters in the TIM_TimeBaseInitStruct.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral
            @arg  PWM0
						@arg  PWM1
  * @param  TimeBaseInit: pointer to a PWM_TimeBaseInitTypeDef
  *         structure that contains the configuration information for
  *         the specified PWM peripheral.
	          @param  PWM_AutoReloadValue: Set the counter reload value.
						        This parameter is between 0 and 0xFFFF.
						@param  PWM_InternalClockSource: Set the PWM clcok.
						        @arg  PWM_InternalClockSource_PCLK
										@arg  PWM_InternalClockSource_SYSCLK
						@param  Prescaler: Set the Prescaler value.
						        This parameter is between 0 and 0xFF.
						@param  PWM_Direction: Select the Counter Mode.
						        @arg PWM_Direction_Up
										@arg PWM_Direction_Down
						@param  PWM_CenterAlignedMode: Select the Counter Mode
						        @arg  PWM_CenterAlignedMode_Disable
										@arg  PWM_CenterAlignedMode_Enable
  * @retval None
  */		
void PMW_TimeBaseInit(PWM_TypeDef* PWMx, PWM_TimeBaseInitTypeDef* TimeBaseInit)
{
  u32  tmpreg = 0;
	
  /* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_InternalClockSource(TimeBaseInit->PWM_InternalClockSource));
	assert_param(IS_PWM_Direction(TimeBaseInit->PWM_Direction));
	assert_param(IS_PWM_CenterAlignedMode(TimeBaseInit->PWM_CenterAlignedMode));
	assert_param(IS_PWM_Prescaler(TimeBaseInit->PWM_Prescaler));
	assert_param(IS_PWM_AutoReloadValue(TimeBaseInit->PWM_AutoReloadValue));

  /* Set the counter reload value */
  PWMx->ARR = TimeBaseInit->PWM_AutoReloadValue;

  /* Set the PWM clcok  */
	tmpreg = PWMx->CR1;
	tmpreg &= ~PWM_CR1_CLKS ;
	tmpreg |= TimeBaseInit->PWM_InternalClockSource;
	PWMx->CR1 = tmpreg;
	
  /* Set the Prescaler value */
  PWMx->PSC = TimeBaseInit->PWM_Prescaler;

  /* Select the Counter Mode */
	tmpreg = 0;
	tmpreg &= ~(PWM_CR2_DIR | PWM_CR2_CMS);
	tmpreg |=  (TimeBaseInit->PWM_Direction | TimeBaseInit->PWM_CenterAlignedMode);
	
  PWMx->CR2 = tmpreg;
}


void PWM_OCInit(PWM_TypeDef *PWMx, PWM_OCInitTypeDef *OutInit)
{
	u8 ch = 0;
	u32 tmpreg1 = 0;
	/* Check the parameters */
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_Channel(OutInit->PWM_Channel));
	assert_param(IS_PWM_OCMode(OutInit->PWM_OCMode));
	assert_param(IS_PWM_OCIdleState(OutInit->PWM_OCIdleState));
	assert_param(IS_PWM_OCValue(OutInit->PWM_OCValue));
	assert_param(IS_PWM_OCNOutput(OutInit->PWM_OCNOutput));

	ch = OutInit->PWM_Channel;
	/* Set the CHx Compare value */
	tmpreg1 = PWMx->OCMR;
	tmpreg1 &= (~Init_OCMR_MASK1(ch)&(~Init_OCMR_MASK2(ch)));

	tmpreg1 |=	((OutInit->PWM_OCMode << (4 * ch))			 | \
							 (OutInit->PWM_OCIdleState << (1 * ch))  | \
							 (OutInit->PWM_OCNOutput << (1 * ch)));
	PWMx->OCMR = tmpreg1;

	PWMx->OCR[ch] = OutInit->PWM_OCValue;
}


/**
  * @brief  Fills each TIM_TimeBaseInitStruct member with its default value.
  * @param  TIM_TimeBaseInitStruct: pointer to a TIM_TimeBaseInitTypeDef structure
  *         which will be initialized.
	          @param  PWM_AutoReloadValue: Set the counter reload value.
						        This parameter is between 0 and 0xFFFF.
						@param  PWM_InternalClockSource: Set the PWM clcok.
						        @arg  PCLK
										@arg  HCLK
						@param  Prescaler: Set the Prescaler value.
						        This parameter is between 0 and 0xFF.
						@param  PWM_Direction: Select the Counter Mode.
						@param  PWM_CenterAlignedMode: Select the Counter Mode
						        @arg  PWM_CounterDirRv_EN
										@arg  PWM_CounterDirRv_DIS
  * @retval None
  */
void PWM_TimeBaseStructInit(PWM_TimeBaseInitTypeDef* TimeBaseInit)
{
  /* Set the default configuration */
  TimeBaseInit->PWM_AutoReloadValue = 0xFFFF;
  TimeBaseInit->PWM_InternalClockSource = PWM_InternalClockSource_PCLK;
  TimeBaseInit->PWM_Prescaler = 0x0000;
  TimeBaseInit->PWM_Direction = PWM_CNT_MODE_UP;
  TimeBaseInit->PWM_CenterAlignedMode = PWM_CenterAlignedMode_Disable;
}

/**
  * @brief  Configures the PWMn Match Reload Times Trigger INT.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  RepeatTimes: specifies the Match Times to Trigger INT
            This parameter is between 0 and 0xF.
  * @retval None
  */
void PWM_INT_RepeatTimes(PWM_TypeDef* PWMx, u8 RepeatTimes)
{
  /* Check the parameters */	
  assert_param(IS_PWM_ALL_PERIPH(PWMx)); 
  assert_param(IS_PWM_REPEATTIME(RepeatTimes) );

  /* Set the PWM INT_RepeatTimes */
	PWMx->ITARR = RepeatTimes;	
}


/**
  * @brief  Configures the PWMx Clock Source.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  PWM_ClockSel: specifies the PWMx Clock Source
  *          This parameter can be one of the following values:
  *         @arg PCLK 
  *         @arg HCLK
  * @note   
  * @retval None
  */
void PWM_ClockSourceConfig(PWM_TypeDef* PWMx, u8 PWM_ClockSel)
{

  u32  tmpreg = 0;
	
  /* Check the parameters */	
  assert_param(IS_PWM_ALL_PERIPH(PWMx)); 
  assert_param(IS_PWM_InternalClockSource(PWM_ClockSel));

  /* Set the PWM Clock Source */
	tmpreg = 	PWMx->CR1;
	tmpreg &= ~PWM_CR1_CLKS ;
	tmpreg |= PWM_ClockSel;
	PWMx->CR1 = tmpreg;	
}

/**
  * @brief  Configures the PWMx PWM_Prescaler.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1 
  * @param  PWM_Prescaler: specifies the PWM_Prescaler Register value
            This parameter is between 0 and 0xFF.
  * @note   The PWM_Prescaler is loaded at the CNTTT update event or immediatly.
  * @retval None
  */
void PWM_PrescalerConfig(PWM_TypeDef* PWMx, u8 PWM_Prescaler)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_PRESCALER(PWM_Prescaler));
  
  /* Set the PWM_Prescaler value */
  PWMx->PSC = PWM_Prescaler;
}


/**
  * @brief  Configures the PWMx Reload value.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1  
  * @param  Reload: specifies the Counter reload value (0~65535)
            This parameter is between 0 and 0xFFFF.
  * @note   The value is loaded at the update event or immediatly
  * @retval None
  */
void PWM_SetCounterReload(PWM_TypeDef* PWMx, u16 Reload)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));	
  assert_param(IS_PWM_COUNTER_RELOAD(Reload));  

  /* Set the counter reload value */
  PWMx->ARR = Reload;
}

/**
  * @brief  Configures the PWMx Current Value.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1      
  * @param  Counter: specifies the Counter register new value.(0~65535)
            This parameter is between 0 and 0xFFFF.
  * @note   The Function is valid when PWM no ouput
  * @retval None
  */
void PWM_SetCounter(PWM_TypeDef* PWMx, u16 Counter)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));	
  assert_param(IS_PWM_COUNTER_VALUE(Counter));  
   
  /* Set the Counter Register value */
  PWMx->CNT = Counter;
}

/**
  * @brief  Gets the PWMx Counter value.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1        
  * @retval Counter Register value.
  */
u16 PWM_GetCounter(PWM_TypeDef* PWMx)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  
  return  PWMx->CNT;
}

/**
  * @brief  Gets the PWMx Current INT RepeatTimes.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1       
  * @retval  Current INT RepeatTimes.
  */
u8 PWM_GetINT_RepeatTimes(PWM_TypeDef* PWMx)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  
  return  PWMx->ITCNT;
}


/**
  * @brief  Gets the PWMx Prescaler value.
  * @param  TIMx: where x can be 0, 1 to select the PWM peripheral.    
  * @retval Prescaler Register value.
  */
u16 PWM_GetPrescaler(PWM_TypeDef* PWMx)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  
  /* Get the Prescaler Register value */
  return  PWMx->PSC;
}


/**
  * @brief  Specifies the PWMx Counter Mode to be used.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1    
  * @param  PWM_Direction: specifies the Counter Mode to be used
  *          This parameter can be one of the following values:
  *            @arg PWM_CNT_MODE_UP: TIM Up Counting Mode
  *            @arg PWM_CNT_MODE_DOWN: TIM Down Counting Mode
  * @param  DirRv_EN: specifies the Counter Direction Reverse
  *            @arg PWM_CounterDirRv_EN: TIM Direction Reverse Enable
  *            @arg PWM_CounterDirRv_DIS: TIM Direction Reverse Disable
  * @retval None
  */
void PWM_CounterModeConfig(PWM_TypeDef* PWMx, u8 PWM_Direction, u8 DirRv_EN)
{
  u32 tmpreg = 0;
  
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Direction(PWM_Direction));	
	assert_param(IS_PWM_CenterAlignedMode(DirRv_EN));
  
  /* Select the Counter Mode */
	tmpreg = PWMx->CR2;
	tmpreg &= ~(PWM_CR2_DIR | PWM_CR2_CMS);
	tmpreg |=  (PWM_Direction | DirRv_EN);
	
  PWMx->CR2 = tmpreg;
}

/**
  * @brief  Update settiongs of the PWMx
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1    
  * @retval None
  */
void PWM_UpdateCmd(PWM_TypeDef* PWMx)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
    /* Set the update bit */
    PWMx->CR1 |= PWM_CR1_UG;
}

/**
  * @brief  Enable or Disable PWMx Counter
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1  
  * @param  NewState: This parameter can be ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void PWM_Cmd(PWM_TypeDef* PWMx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	

  if (NewState == ENABLE)
  {
    /* Set the update bit */
    PWMx->CR1 |= PWM_CR1_EN;
  }
  else
  {
    /* Reset the update bit */
    PWMx->CR1 &= (~PWM_CR1_EN);
  }
}

/**
  * @brief  Enable or Disable PWMx brake function
   * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1  
  * @param  NewState: This parameter can be ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void PWM_BreakInput_Cmd(PWM_TypeDef* PWMx, FunctionalState NewState)
{		
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
		
  if (NewState != DISABLE)
  {
    /* Enable BreakInput*/
		PWMx->OCMR |= PWM_OCMR_BKE;
  }
  else
  {
    /* Disable BreakInput*/
		PWMx->OCMR &= ~PWM_OCMR_BKE;  
  }	
}

/**
  * @brief  Control the effective level of brake input.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1  
  * @param  BKI_Level: This parameter can be PWM_BreakPolarity_High or PWM_BreakPolarity_Low.
            @arg  PWM_BreakPolarity_High
						@arg  PWM_BreakPolarity_Low 
  * @retval None
  */
void PWM_BKI_LevelConfig(PWM_TypeDef* PWMx, u32 BKI_Level)
{
	u32 wTmpReg = 0;	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_BreakPolarity(PWM_BreakPolarity_High));
	
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~PWM_OCMR_BKP;
	
	wTmpReg |=BKI_Level;
	
	PWMx->OCMR = wTmpReg;
		

}

/**
  * @brief  Enable or Disable the interruption of matching, capturing and braking.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  PWM_INT: Specify the PWM interrupts sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *        @arg PWM_OCIT_ARI      : PWM ARR Reload value match interrupt
  *        @arg PWM_OCIT_OC1I      : PWM CH1 MR1 vaule match interrupt
  *        @arg PWM_OCIT_OC2I      : PWM CH2 MR2 vaule match interrupt
  *        @arg PWM_OCIT_OC3I      : PWM CH3 MR3 vaule match interrupt
  *        @arg PWM_OCIT_OC4I      : PWM CH4 MR4 vaule match interrupt
  *        @arg PWM_ICIT_IC1I     : PWM CH1 Capture interrupt
  *        @arg PWM_ICIT_IC2I     : PWM CH2 Capture interrupt
  *        @arg PWM_ICIT_IC3I     : PWM CH3 Capture interrupt
  *        @arg PWM_ICIT_IC4I     : PWM CH4 Capture interrupt
  *        @arg PWM_BKIT_BKI       : PWM break interrupt
  * @param  NewState: This parameter can be ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void PWM_InterruptConfig(PWM_TypeDef* PWMx, u32 PWM_INT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Interrput(PWM_INT));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));
	
	if((PWM_INT==PWM_OCIT_ARI) | (PWM_INT==PWM_OCIT_OC1I) | (PWM_INT==PWM_OCIT_OC2I) | (PWM_INT==PWM_OCIT_OC3I)| (PWM_INT==PWM_OCIT_OC4I))
	{
    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      PWMx->CR2 |= PWM_INT;
    }
    else
    {
      /* Disable the interrupt sources */
      PWMx->CR2 &= ~PWM_INT;
    }
  }
	
	if(PWM_INT==PWM_BKIT_BKI)
	{
    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      PWMx->DTCR |= PWM_INT;
    }
    else
    {
      /* Disable the interrupt sources */
      PWMx->DTCR &= ~PWM_INT;
    }
  }	
	
}


void PWM_CapITConfig(PWM_TypeDef* PWMx,u32 PWM_CAP_INT,FunctionalState NewState)
{
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_CAP_INT(PWM_CAP_INT));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));
	
	
	if((PWM_CAP_INT==PWM_ICIT_IC1I) | (PWM_CAP_INT==PWM_ICIT_IC2I) | (PWM_CAP_INT==PWM_ICIT_IC3I) | (PWM_CAP_INT==PWM_ICIT_IC4I))
	{
    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      PWMx->CAPR |= PWM_CAP_INT;
    }
    else
    {
      /* Disable the interrupt sources */
      PWMx->CAPR &= ~PWM_CAP_INT;
    }
  }
}

/**
  * @brief  Check the flag bit of PWM.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  PWM_FLAG: Specify the flag to be checked.
  *         This parameter can be one of the following values:
  *         @arg PWM_FLAG_ARF       : PWM ARR Reload value match flag
  *         @arg PWM_FLAG_OC1F       : PWM CH1 MR1 vaule match flag
  *         @arg PWM_FLAG_OC2F       : PWM CH2 MR2 vaule match flag
  *         @arg PWM_FLAG_OC3F       : PWM CH3 MR3 vaule match flag
  *         @arg PWM_FLAG_OC4F       : PWM CH4 MR4 vaule match flag
  *         @arg PWM_FLAG_IC1R   : PWM CH1 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC1F   : PWM CH1 Capture Falling Edge flag
  *         @arg PWM_FLAG_IC2R   : PWM CH2 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC2F   : PWM CH2 Capture Falling Edge flag
  *         @arg PWM_FLAG_IC3R   : PWM CH3 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC3F   : PWM CH3 Capture Falling Edge flag
  *         @arg PWM_FLAG_IC4R   : PWM CH4 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC4F   : PWM CH4 Capture Falling Edge flag
  *         @arg PWM_FLAG_BK        : PWM break flag
  * @return FlagStatus of PWM_FLAG (SET or RESET).
  */
FlagStatus PWM_GetFlagStatus(PWM_TypeDef* PWMx, u32 PWM_FLAG)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_PWM_FLAG(PWM_FLAG));	

  if ((PWMx->SR & PWM_FLAG) != 0)
  {
    return SET;
  }
  else
  {
    return RESET;
  }
}

/**
  * @brief  Clear the flag bit of PWM.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  PWM_FLAG: Specify the flag to be cleared.
  *         This parameter can be one of the following values:
  *         @arg PWM_FLAG_ARF       : PWM ARR Reload value match flag
  *         @arg PWM_FLAG_OC1F       : PWM CH1 MR1 vaule match flag
  *         @arg PWM_FLAG_OC2F       : PWM CH2 MR2 vaule match flag
  *         @arg PWM_FLAG_OC3F       : PWM CH3 MR3 vaule match flag
  *         @arg PWM_FLAG_OC4F       : PWM CH4 MR4 vaule match flag
  *         @arg PWM_FLAG_IC1R   : PWM CH1 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC1F   : PWM CH1 Capture Falling Edge flag
  *         @arg PWM_FLAG_IC2R   : PWM CH2 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC2F   : PWM CH2 Capture Falling Edge flag
  *         @arg PWM_FLAG_IC3R   : PWM CH3 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC3F   : PWM CH3 Capture Falling Edge flag
  *         @arg PWM_FLAG_IC4R   : PWM CH4 Capture Rising  Edge flag
  *         @arg PWM_FLAG_IC4F   : PWM CH4 Capture Falling Edge flag
  *         @arg PWM_FLAG_BK        : PWM break flag
  * @retval None
  */
void PWM_ClearFlag(PWM_TypeDef* PWMx, u32 PWM_FLAG)
{
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_CLEAR_FLAG(PWM_FLAG));	
	
  /* Clear the flags */
  PWMx->SR = PWM_FLAG;
}


/**
  * @brief  Enable or Disable PWMx CHx RisingEdge Capture
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1 
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1       : 
  *         @arg PWM_Channel_2       : 
  *         @arg PWM_Channel_3       : 
  *         @arg PWM_Channel_4       : 
  * @param  NewState: This parameter can be ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void PWM_RE_CaptureConfig(PWM_TypeDef* PWMx, u8 CHx, FunctionalState NewState)
{
	
	u32 wTmpReg = 0;
	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
	
	wTmpReg = PWMx->CAPR;
	
	wTmpReg &= ~(PWM_CAPR_IC1R <<CHx*4);
	
  if (NewState != DISABLE)
  {
    /* Enable RisingEdge Capture*/
		wTmpReg |= (PWM_CAPR_IC1R <<CHx*4);
  }
  else
  {
    /* Disable RisingEdge Capture*/
		wTmpReg |= 0;    
  }	
	  PWMx->CAPR = wTmpReg;
}


/**
  * @brief  Enable or Disable PWMx CHx FallingEdge Capture.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.  
            @arg  PWM0
						@arg  PWM1 
  * @param  CHx:  Specify the Channel to be selected.  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1       : 
  *         @arg PWM_Channel_2       : 
  *         @arg PWM_Channel_3       : 
  *         @arg PWM_Channel_4       : 
  * @param  NewState: This parameter can be ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void PWM_FE_CaptureConfig(PWM_TypeDef* PWMx, u8 CHx, FunctionalState NewState)
{
	
	u32 wTmpReg = 0;
	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
	
	wTmpReg = PWMx->CAPR;
	wTmpReg &= ~(PWM_CAPR_IC1F <<CHx*4);
	
  if (NewState != DISABLE)
  {
    /* Enable FallingEdge Capture*/
		wTmpReg |= (PWM_CAPR_IC1F <<CHx*4);
  }
  else
  {
    /* Disable FallingEdge Capture*/
		wTmpReg |= 0;    
  }	
	  PWMx->CAPR = wTmpReg;
}

/**
  * @brief  Enable or Disable PWMx CHx Capture Rester
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  CHx:  Specify the Channel to be selected . 
  *         This parameter can be one of the following values:
  *         @arg  PWM_Channel_1      
  *         @arg  PWM_Channel_2      
  *         @arg  PWM_Channel_3      
  *         @arg  PWM_Channel_4     
  * @param  NewState: This parameter can be ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void PWM_RST_CaptureConfig(PWM_TypeDef* PWMx, u8 CHx, FunctionalState NewState)
{
	
	u32 wTmpReg = 0;
	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
	
	wTmpReg = PWMx->CAPR;
	wTmpReg &= ~(PWM_CAPR_IC1RC <<CHx*4);
	
  if (NewState != DISABLE)
  {
    /* Enable Capture Reset*/
		wTmpReg |= (PWM_CAPR_IC1RC <<CHx*4);
  }
  else
  {
    /* Disable Capture Reset*/
		wTmpReg |= 0;    
  }	
	  PWMx->CAPR = wTmpReg;
}


/**
  * @brief  Gets the PWMx CHx Capture Counter value.
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1     
  *         @arg PWM_Channel_2     
  *         @arg PWM_Channel_3      
  *         @arg PWM_Channel_4    
  * @retval Change time value.
  */
u16 PWM_GetCaptureCounter(PWM_TypeDef* PWMx, u8 CHx)
{
	u16 capture_cnt = 0;
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  
  /* Get the Capture Counter Register value */
	if(CHx==PWM_Channel_1)
	capture_cnt = PWMx->ICR[0];
	if(CHx==PWM_Channel_2)
	capture_cnt = PWMx->ICR[1];
	if(CHx==PWM_Channel_3)
	capture_cnt = PWMx->ICR[2];
	if(CHx==PWM_Channel_4)
	capture_cnt = PWMx->ICR[3];
		
  return  capture_cnt;
}

/**
  * @brief  Gets the PWMx CHx Capture Edge .
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1  
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1      
  *         @arg PWM_Channel_2       
  *         @arg PWM_Channel_3     
  *         @arg PWM_Channel_4      
  * @retval  Capture Edge.
  *         This parameter can be one of the following values:
  *         @arg CAPTURE_FE     :  capture falling edge 
  *         @arg CAPTURE_RE     :  capture rising  edge  
  */
u16 PWM_GetCaptureEdge(PWM_TypeDef* PWMx, u8 CHx)
{
	u16 capture_edge = 0;
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  
  /* Get the Capture Counter Register value */
	if(CHx==PWM_Channel_1)
	capture_edge = (PWMx->ICR[0] >> 16);
	if(CHx==PWM_Channel_2)
	capture_edge = (PWMx->ICR[1] >> 16) ;
	if(CHx==PWM_Channel_3)
	capture_edge = (PWMx->ICR[2] >> 16) ;
	if(CHx==PWM_Channel_4)
	capture_edge = (PWMx->ICR[3] >> 16) ;
		
  return  capture_edge;
}


/**
  * @brief  Set the PWMx CHx Output Mode Config
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1 
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1        : 
  *         @arg PWM_Channel_2        : 
  *         @arg PWM_Channel_3        : 
  *         @arg PWM_Channel_4        : 
  * @param  Mode: Specify the Channel output mode  
  *         This parameter can be one of the following values:
  *         @arg NO_OUTPUT       :  PWM channel no output
  *         @arg COMPAER_MODE1   :  PWM channel output high when tc <= compare value,output low when tc > compare value 
  *         @arg COMPAER_MODE2   :  PWM channel output low when tc <= compare value,output high when tc > compare value
  *         @arg COMPAER_MODE3   :  PWM channel output reverse when tc match compare value
  * @retval None
  */
void PWM_OD_Config(PWM_TypeDef* PWMx, u8 CHx, u32 Mode)
{
	
	u32 wTmpReg = 0;
	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  assert_param(IS_PWM_OCMode(Mode));	
	
	wTmpReg = PWMx->OCMR;
	wTmpReg &= ~(PWM_OCMR_OC1M <<CHx*4);
	wTmpReg |= (Mode <<CHx*4);
	
	/* Set PWMx CH Output Mode*/

	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Set the PWMx CHx Output idle
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1  
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1        : 
  *         @arg PWM_Channel_2        : 
  *         @arg PWM_Channel_3        : 
  *         @arg PWM_Channel_4        : 
  * @param  Idle: Specify the Channel output mode  
  *         This parameter can be one of the following values:
  *         @arg PWM_IDLE_LOW    :  PWM channel idle is low
  *         @arg PWM_IDLE_HIGH   :  PWM channel idle is high
  * @retval None
  */
void PWM_IDLE_Config(PWM_TypeDef* PWMx, u8 CHx, u32 Idle)
{
	
	u32 wTmpReg = 0;
	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  assert_param(IS_PWM_IDLE(Idle));	
	
	wTmpReg = PWMx->OCMR;	
	wTmpReg &= ~(PWM_OCMR_OIS1 <<CHx);
	wTmpReg |= (Idle <<CHx);
	
	/* Set PWMx CH Output Idle*/
	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Enable or Disable the PWMx CHx Complementary Output
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1  
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1        : 
  *         @arg PWM_Channel_2        : 
  *         @arg PWM_Channel_3        : 
  *         @arg PWM_Channel_4        : 
  * @param  CHNCTL: Specify the Channel output mode  
  *         This parameter can be one of the following values:
  *         @arg PWM_CHNCTL_DISABLE  :  Disable PWMx CHx Complementary Output
  *         @arg PWM_CHNCTL_ENABLE   :  Enable PWMx CHx Complementary Output
  * @retval None
  */
void PWM_CHN_Config(PWM_TypeDef* PWMx, u8 CHx, u32 CHNCTL)
{
	
	u32 wTmpReg = 0;
	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_Channel(CHx));
  assert_param(IS_PWM_CHNCTL(CHNCTL));	
	
	wTmpReg = PWMx->OCMR;	
	wTmpReg &= ~(PWM_OCMR_OC1NE <<CHx);
	wTmpReg |= (CHNCTL <<CHx);
	
	/* Set PWMx CH Complementary Output*/
	PWMx->OCMR = wTmpReg;
}

/**
  * @brief  Set the PWMx Death Time
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.
            @arg  PWM0
						@arg  PWM1  
  * @param  DeathTime: Specify the Death value (0~0xFF)  
  * @retval None
  */
void PWM_DT_Config(PWM_TypeDef* PWMx, u8 CHx, u8 DeathTime)
{

	
  /* Check the parameters */
  assert_param(IS_PWM_ALL_PERIPH(PWMx));
  assert_param(IS_PWM_DEATHTIME(DeathTime));	
	
	
	/* Set PWMx Death Time*/
	PWMx->DTCR = DeathTime;
}

/**
  * @brief  Set the PWMx CHx Compare Value
  * @param  PWMx: where x can be 0, 1 to select the PWM peripheral.    
  * @param  CHx:  Specify the Channel to be selected  
  *         This parameter can be one of the following values:
  *         @arg PWM_Channel_1        : 
  *         @arg PWM_Channel_2        : 
  *         @arg PWM_Channel_3        : 
  *         @arg PWM_Channel_4        : 
  * @param  Cmp: Specify the CHx Output Compare Value value (0~0xFFFF)  
  * @retval None
  */
void PWM_SetOCxValue(PWM_TypeDef* PWMx, u8 CHx, u16 value)
{

	/* Check the parameters */
	u8 ch = 0;
	assert_param(IS_PWM_ALL_PERIPH(PWMx));
	assert_param(IS_PWM_Channel(CHx));
	assert_param(IS_PWM_OCValue(value));
	ch = CHx;
	PWMx->OCR[ch] = value;
}


/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
