/**
  ******************************************************************************
  * @file    PT32x031x_rcc.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1 
  * @brief   This file provides firmware functions to manage the following 
  *          
  *
 @verbatim
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_rcc.h"

#define  HSI_VALUE     24000000UL
#define  HSE_VALUE     24000000UL
#define  LSI_VALUE     32768UL

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup RCC 
  * @brief RCC driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup RCC_Private_Functions
  * @{
  */


void RCC_AutomaticAmplitude(FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    /* Enable Automatic amplitude adjustment of external crystal oscillator */
    RCC->HSECR1 |= RCC_HSECR1_ACC;
  }
  else
  {
    /* Disable Automatic amplitude adjustment of external crystal oscillator */
    RCC->HSECR1 &= (u32)~RCC_HSECR1_ACC;
  }
}


/**
  * @brief  Gets the value of clock frequence.
  * @param  RCC_Clock: specify the clock selection.
	*        @arg RCC_SYSCLK: Get the system clock frequence
	*        @arg RCC_HCLK: Get the AHB clock frequence
	*        @arg RCC_PCLK: Get the APB clock frequence
	*        @arg RCC_HSI: Get the HSI	clock frequence
	*        @arg RCC_LSI: Get the LSI clock frequence
	*        @arg RCC_LSE: Get the LSE clock frequence
  * @retval Clock Frequent value
  */
u32 RCC_GetClockFreq(u8 RCC_Clock)
{
	u32 system_clock=0,pclk_clock=0,hclk_clock=0;
	/* Check the parameters */
	assert_param(IS_RCC_Clock(RCC_Clock));

	/*if system clock source is HSI*/
	if((RCC->CFGR&RCC_CFGR_SCW)==RCC_HSI)
	{
		system_clock=HSI_VALUE;
	}
	/*if system clock source is HSE*/
	else if((RCC->CFGR&RCC_CFGR_SCW)==RCC_HSE)
	{
		system_clock=HSE_VALUE;
	}
	/*if system clock source is PLL*/
	else if((RCC->CFGR&RCC_CFGR_SCW)==RCC_PLL)
	{
		if((RCC->CFGR&RCC_CFGR_PLLSRC)==0x0)
		{
			system_clock=(u32)(HSI_VALUE*2);
		}
		else if((RCC->CFGR&RCC_CFGR_PLLSRC)==0x4)
		{
			system_clock=(u32)(HSE_VALUE*2);
		}			
		else
		{
			system_clock=(u32)(LSI_VALUE*2);
		}
	}
	/*if got a error value*/
	else 
	{
		while(1);
	}
	
	hclk_clock=(u32)(system_clock>>((RCC->CFGR&RCC_CFGR_HPRE)>>4));
	pclk_clock=(u32)(hclk_clock>>((RCC->CFGR&RCC_CFGR_PPRE)>>8));
	
	switch (RCC_Clock)
	{
		case RCC_SysClock:
			return (u32)system_clock;
		case RCC_HCLK:
			return (u32)hclk_clock;
		case RCC_PCLK:
			return (u32)pclk_clock;
		default:
			return 0;
	}
}


/**
  * @brief  Reset Remap Bit.
  * @param  None
  * @retval None
  */
void System_Retriming(void)
{
  RCC->ASRCR = KEY_RETRIMING;
}



/** @defgroup RSTCU_Private_Function RSTCU private functions
  * @{
  */

/*********************************************************************************************************//**
 * @brief Check whether the specific global reset flag is set or not.
 * @param reset_flag: specify the reset flag.
 * @retval SET or RESET
 ************************************************************************************************************/
FlagStatus RCC_GetResetFlagStatus(u32 reset_flag)
{
  u32 tmp;

  /* Check the parameters */
  assert_param(IS_RCC_ResetStatusFlag(reset_flag));

  tmp = (RCC->RSR & reset_flag);
  if (tmp != RESET)
  {
    return SET;
  }
  else
  {
    return RESET;
  }
}

/*********************************************************************************************************//**
 * @brief Clear the specific global reset flag.
 * @param reset_flag: specify the reset flag.
 *        This parameter can be:
 * @retval None
 ************************************************************************************************************/
void RCC_ClearResetFlag(u32 reset_flag)
{
  /* Check the parameters */
  assert_param(IS_RCC_ResetStatusFlag(reset_flag));

  RCC->RSR = reset_flag;  /* Write 1 to clear */
}

/*********************************************************************************************************//**
 * @brief Clear all of the global reset flag.
 * @retval None
 ************************************************************************************************************/
void System_ClearAllResetFlag(void)
{
  RCC->RSR = (u32)0xFF ;   /* Write 1 to clear */
}


/**
  * @brief  Enables or disables the specified Reset Source.

  * @param  RstSource: specifies the Reset source to be enabled or disabled. 
  *          This parameter can be one of the following values:
  *            @arg 
  *            @arg 
  *            @arg 
  *            @arg 
  * @param  NewState: new state of the specified SPI interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RCC_ResetConfig(u32 RstSource, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_RCC_ResetEnable(RstSource));

  if (NewState != DISABLE)
  {
    /* Enable the selected SPI interrupt */
    RCC->RCR |= RstSource;
  }
  else
  {
    /* Disable the selected SPI interrupt */
    RCC->RCR &= (uint16_t)~RstSource;
  }
}

/**
  * @brief  Enables or disables the specified clock source
  * @param  ClockSource: specifies the clock source to be enabled or disabled.
  *         This parameter can be one of the following values:
  *         @arg RCC_PLL: Specify the new state of Phase Locked Loop
  *         @arg RCC_HSI: Specify the new state of Internal High Crystal
  *         @arg RCC_LSI: Specify the new state of Internal Low Crystal
	* @param  NewState: new state of the specified Reset Source
  *          This parameter can be: ENABLE or DISABLE.
  *         @return None
  */
void RCC_ClockSourceConfig(u32 ClockSource, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_RCC_ClockSource(ClockSource));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if(ClockSource==RCC_HSI)
	{
		if (NewState != DISABLE)
		{
			RCC->HSICR 	|= RCC_HSICR_EN;
		}
		else
		{
			RCC->HSICR 	&= ~RCC_HSICR_EN;
		}
	}
	else if(ClockSource==RCC_LSI)
	{
		if (NewState != DISABLE)
		{
			RCC->LSICR 	|= RCC_LSICR_EN;
		}
		else
		{
			RCC->LSICR 	&= ~RCC_LSICR_EN;
		}
	}
	else if(ClockSource==RCC_HSE)
	{
		if (NewState != DISABLE)
		{
			RCC->HSECR1 |= RCC_HSECR1_EN;
		}
		else
		{
			RCC->HSECR1 &= ~RCC_HSECR1_EN;
		}	
	}		
		else if(ClockSource==RCC_PLL)
	{
		if (NewState != DISABLE)
		{
			RCC->PCR 	|= RCC_PCR_EN;
		}
		else
		{
			RCC->PCR 	&= ~RCC_PCR_EN;
		}	
	}
	else
	{
		if (NewState != DISABLE)
		{
			RCC->HSECR1 	|= RCC_HSECR1_ECEN;
		}
		else
		{
			RCC->HSECR1 	&= ~RCC_HSECR1_ECEN;
		}	
	}
}
/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
