/**
  ******************************************************************************
  * @file    PT32x031x_spi.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1    
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Serial peripheral interface (SPI):
  *           + Initialization and Configuration
  *           + Data transfers functions
  *           + Interrupts and flags management
  *
  ******************************************************************************
  * @attention
  *
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_spi.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup SPI
  * @brief SPI driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* SPI registers Masks */
#define CR0_CLEAR_MASK       ((u16)0x0030)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup SPI_Private_Functions
  * @{
  */

/** @defgroup SPI_Group1 Initialization and Configuration functions
 *  @brief   Initialization and Configuration functions 
 *
  * @{
  */

/**
  * @brief  Fills each SPI_InitStruct member with its default value.
  * @param  SPI_InitStruct: pointer to a SPI_InitTypeDef structure which will be initialized.
            @param  SPI_Mode: Master or slave mode selection.
                    @arg  SPI_Mode_Master
                    @arg  SPI_Mode_Slave
            @param  SPI_DataSize: Frame data length.
                    @arg  SPI_DataSize_4b
                    @arg  SPI_DataSize_5b
                    @arg  SPI_DataSize_6b
                    @arg  SPI_DataSize_7b
                    @arg  SPI_DataSize_8b
                    @arg  SPI_DataSize_9b
                    @arg  SPI_DataSize_10b
                    @arg	SPI_DataSize_11b
                    @arg  SPI_DataSize_12b
                    @arg  SPI_DataSize_13b
                    @arg  SPI_DataSize_14b
                    @arg  SPI_DataSize_15b
                    @arg	SPI_DataSize_16b									
            @param  SPI_CPOL: Idle state hold level.
						        @arg	SPI_CPOL_Low
										@arg	SPI_CPOL_High
            @param  SPI_CPHA: SPI clock polarity phase control.
						        @arg	SPI_CPHA_1Edge
										@arg	SPI_CPHA_2Edge
            @param  SPI_NSS: Software chip selection signal control.
						        @arg	SPI_NSS_Soft
										@arg	SPI_NSS_Hard						        
            @param  SPI_BaudRatePrescaler: SPI clock prescaler coefficient.
						        The parameter is the even between 1 and 255											
            @param  SPI_BaudRatePostPrescaler: SPI clock division coefficient.
						        The parameter is between 1 and 255
  * @retval None
  */
void SPI_StructInit(SPI_InitTypeDef* SPI_InitStruct)
{
/*--------------- Reset SPI init structure parameters values -----------------*/
  /* Initialize the SPI_Mode member */
  SPI_InitStruct->SPI_Mode = SPI_Mode_Slave;
  /* Initialize the SPI_DataSize member */
  SPI_InitStruct->SPI_DataSize = SPI_DataSize_8b;
  /* Initialize the SPI_CPOL member */
  SPI_InitStruct->SPI_CPOL = SPI_CPOL_Low;
  /* Initialize the SPI_CPHA member */
  SPI_InitStruct->SPI_CPHA = SPI_CPHA_1Edge;
  /* Initialize the SPI_NSS member */
  SPI_InitStruct->SPI_NSS = SPI_NSS_Hard;
  /* Initialize the SPI_BaudRatePrescaler member */
  SPI_InitStruct->SPI_BaudRatePrescaler = 2;
  /* Initialize the SPI_BaudRatePrescaler member */	
	SPI_InitStruct->SPI_BaudRatePostPrescaler = 0;
}

/**
  * @brief  Initializes the SPIx peripheral according to the specified 
  *         parameters in the SPI_InitStruct.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  SPI_InitStruct: pointer to a SPI_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
            @param  SPI_Mode: Master or slave mode selection.
                    @arg  SPI_Mode_Master
                    @arg  SPI_Mode_Slave
            @param  SPI_DataSize: Frame data length.
                    @arg  SPI_DataSize_4b
                    @arg  SPI_DataSize_5b
                    @arg  SPI_DataSize_6b
                    @arg  SPI_DataSize_7b
                    @arg  SPI_DataSize_8b
                    @arg  SPI_DataSize_9b
                    @arg  SPI_DataSize_10b
                    @arg	SPI_DataSize_11b
                    @arg  SPI_DataSize_12b
                    @arg  SPI_DataSize_13b
                    @arg  SPI_DataSize_14b
                    @arg  SPI_DataSize_15b
                    @arg	SPI_DataSize_16b									
            @param  SPI_CPOL: Idle state hold level.
						        @arg	SPI_CPOL_Low
										@arg	SPI_CPOL_High
            @param  SPI_CPHA: SPI clock polarity phase control.
						        @arg	SPI_CPHA_1Edge
										@arg	SPI_CPHA_2Edge
            @param  SPI_NSS: Software chip selection signal control.
						        @arg	SPI_NSS_Soft
										@arg	SPI_NSS_Hard						        
            @param  SPI_BaudRatePrescaler: SPI clock prescaler coefficient.
						        The parameter is the even between 1 and 255						
            @param  SPI_BaudRatePostPrescaler: SPI clock division coefficient.
						        The parameter is between 1 and 255
  * @retval None
  */
void SPI_Init(SPI_TypeDef* SPIx, SPI_InitTypeDef* SPI_InitStruct)
{
  u16 tmpreg = 0;

  /* check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));

  /* Check the SPI parameters */
  assert_param(IS_SPI_MODE(SPI_InitStruct->SPI_Mode));
  assert_param(IS_SPI_DATA_SIZE(SPI_InitStruct->SPI_DataSize));
  assert_param(IS_SPI_CPOL(SPI_InitStruct->SPI_CPOL));
  assert_param(IS_SPI_CPHA(SPI_InitStruct->SPI_CPHA));
  assert_param(IS_SPI_NSS(SPI_InitStruct->SPI_NSS));
  assert_param(IS_SPI_BAUDRATE_PRESCALER(SPI_InitStruct->SPI_BaudRatePrescaler));
	assert_param(IS_SPI_BAUDRATE_POSTPRESCALER(SPI_InitStruct->SPI_BaudRatePostPrescaler));

  /*---------------------------- SPIx CR1 Configuration ------------------------*/
  /* Get the SPIx CR1 value */
  tmpreg = SPIx->CR1;
  /* Clear DSS,CPOL,CPHA and post BaudRatePrescaler bits */
  tmpreg &= CR0_CLEAR_MASK;
  /* Configure SPIx: Data frame length,CPOL and CPHA, SPI_BaudRatePostPrescaler,BaudRate prescaler,
	NSS management,master/slave mode*/
  /* Set Data frame length */
	/* Set CPOL bit according to SPI_CPOL value */
  /* Set CPHA bit according to SPI_CPHA value */
  /* Set SCR  bits according to SPI_BaudRatePostPrescaler value */
  /* Set MSTR bit  according to Master-slave mode */	
  /* Set CSS bits according to SPI_BaudRatePrescaler value */
  /* Set CPOL bit according to SPI_CPOL value */
  /* Set CPHA bit according to SPI_CPHA value */
  tmpreg |= (u16)((u32)SPI_InitStruct->SPI_DataSize | SPI_InitStruct->SPI_CPOL |
                      SPI_InitStruct->SPI_CPHA | ((SPI_InitStruct->SPI_BaudRatePostPrescaler)<<8));  
  /* Write to SPIx CR1 */
  SPIx->CR1 = tmpreg;
  /*-------------------------SPI Mode Configuration -----------------------*/
  /* Clear MS bits */
  SPIx->CR2 &=(u16)~SPI_CR2_MSM;
  /* Configure SPIx: Mode */
  SPIx->CR2 |= (u16)(SPI_InitStruct->SPI_Mode); 
  /*---------------------------- SPIx CPSR Configuration --------------------*/
  /* Write to SPIx Prescaler */
  SPIx->BR = SPI_InitStruct->SPI_BaudRatePrescaler;
	
	tmpreg = SPIx->CSS;
	tmpreg &= ~SPI_CSS_CSS;
	tmpreg |= SPI_InitStruct->SPI_NSS;
	SPIx->CSS = tmpreg;
  
}


/**
  * @brief  Enables or disables the specified SPI peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  NewState: new state of the SPIx peripheral. 
  *          This parameter can be: ENABLE or DISABLE.
            @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void SPI_Cmd(SPI_TypeDef* SPIx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the selected SPI peripheral */
    SPIx->CR2 |= SPI_CR2_EN;
  }
  else
  {
    /* Disable the selected SPI peripheral */
    SPIx->CR2 &= (u16)~((u16)SPI_CR2_EN);
  }
}

/**
  * @brief  Configures the data size for the selected SPI.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  SPI_DataSize: specifies the SPI data size.
  *         For the SPIx peripheral this parameter can be one of the following values:   
  *            @arg SPI_DataSize_4b: Set data size to 4 bits
  *            @arg SPI_DataSize_5b: Set data size to 5 bits
  *            @arg SPI_DataSize_6b: Set data size to 6 bits
  *            @arg SPI_DataSize_7b: Set data size to 7 bits
  *            @arg SPI_DataSize_8b: Set data size to 8 bits
  *            @arg SPI_DataSize_9b: Set data size to 9 bits
  *            @arg SPI_DataSize_10b: Set data size to 10 bits
  *            @arg SPI_DataSize_11b: Set data size to 11 bits
  *            @arg SPI_DataSize_12b: Set data size to 12 bits
  *            @arg SPI_DataSize_13b: Set data size to 13 bits
  *            @arg SPI_DataSize_14b: Set data size to 14 bits
  *            @arg SPI_DataSize_15b: Set data size to 15 bits
  *            @arg SPI_DataSize_16b: Set data size to 16 bits
  * @retval None
  */
void SPI_DataSizeConfig(SPI_TypeDef* SPIx, u16 SPI_DataSize)
{
  u16 tmpreg = 0;
  
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  assert_param(IS_SPI_DATA_SIZE(SPI_DataSize));
  /* Read the CR2 register */
  tmpreg = SPIx->CR1;
  /* Clear DS[3:0] bits */
  tmpreg &= (u16)~SPI_CR1_DFF;
  /* Set new DS[3:0] bits value */
  tmpreg |= SPI_DataSize;
  SPIx->CR1 = tmpreg;
}


/**
  * @brief  Configures internally by software the NSS pin for the selected SPI.
  * @note   This function can be called only after the SPI_Init() function has 
  *         been called.  
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  SPI_NSSInternalSoft: specifies the SPI NSS internal state.
  *          This parameter can be one of the following values:
  *            @arg SPI_NSSInternalSoft_Set: Set NSS pin internally
  *            @arg SPI_NSSInternalSoft_Reset: Reset NSS pin internally
  * @retval None
  */
void SPI_NSSInternalSoftwareConfig(SPI_TypeDef* SPIx, u16 SPI_NSSInternalSoft)
{
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  assert_param(IS_SPI_NSS_INTERNAL(SPI_NSSInternalSoft));

  if (SPI_NSSInternalSoft != SPI_NSSInternalSoft_Reset)
  {
    /* Set NSS pin internally by software */
    SPIx->CSS |= SPI_NSSInternalSoft_Set;
  }
  else
  {
    /* Reset NSS pin internally by software */
    SPIx->CSS &= SPI_NSSInternalSoft_Reset;
  }
}

/**
  * @brief  Transmits a Data through the SPIx/I2Sx peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  Data: Data to be transmitted.
  * @retval None
  */
void SPI_SendData8(SPI_TypeDef* SPIx, u8 Data)
{
  u32 spixbase = 0x00;

  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));

  spixbase = (u32)SPIx; 
  spixbase += 0x08;
  
  *(__IO u8 *) spixbase = Data;
}

/**
  * @brief  Returns the most recent received data by the SPIx/I2Sx peripheral. 
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @retval The value of the received data.
  */
u8 SPI_ReceiveData8(SPI_TypeDef* SPIx)
{
  u32 spixbase = 0x00;
  
  spixbase = (u32)SPIx; 
  spixbase += 0x08;
  
  return *(__IO u8 *) spixbase;
}

/**
  * @brief  Transmits a Data through the SPIx/I2Sx peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  Data : Data to be transmitted.
  * @retval None
  */
void SPI_SendData(SPI_TypeDef* SPIx, u16 Data)
{
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  
  /* Write in the DR register the data to be sent */
  SPIx->DR = Data;
}

/**
  * @brief  Returns the most recent received data by the SPIx/I2Sx peripheral. 
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @retval The value of the received data.
  */
u16 SPI_ReceiveData(SPI_TypeDef* SPIx)
{
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  
  /* Return the data in the DR register */
  return SPIx->DR;
}


/**
  * @brief  Enables or disables the specified SPI interrupts.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  SPI_IT: specifies the SPI interrupt source to be enabled or disabled. 
  *          This parameter can be one of the following values:
  *            @arg SPI_IT_RFOF: Rx buffer overflow interrupt mask
  *            @arg SPI_IT_RFNEOT: Rx buffer not empty interrupt mask
  *            @arg SPI_IT_RFHF: Rx buffer Half-full interrupt mask
  *            @arg SPI_IT_TFHF: Tx buffer Half-full interrupt mask	
  * @param  NewState: new state of the specified SPI interrupt.
  *          This parameter can be: ENABLE or DISABLE.
	          @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void SPI_ITConfig(SPI_TypeDef* SPIx, u8 SPI_IT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_SPI_CONFIG_IT(SPI_IT));

  if (NewState != DISABLE)
  {
    /* Enable the selected SPI interrupt */
    SPIx->IE |= SPI_IT;
  }
  else
  {
    /* Disable the selected SPI interrupt */
    SPIx->IE &= (u16)~SPI_IT;
  }
}



/**
  * @brief  Checks whether the specified SPI flag is set or not.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1  
  * @param  SPI_FLAG: specifies the SPI flag to check. 
  *          This parameter can be one of the following values:
  *            @arg SPI_FLAG_TFE: Transmit buffer empty flag.
  *            @arg SPI_FLAG_TFNF: Transmit buffer non-full flag.
  *            @arg SPI_FLAG_BUSY:  Transmission module is busy.    
  *            @arg SPI_FLAG_RFNE: Receive bufer non-empty flag 
  *            @arg SPI_FLAG_RFF:  Receive bufer full flag . 
  * @retval The new state of SPI_FLAG (SET or RESET).
  */
FlagStatus SPI_GetFlagStatus(SPI_TypeDef* SPIx, u16 SPI_FLAG)
{
  FlagStatus bitstatus = RESET;
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  assert_param(IS_SPI_GET_FLAG(SPI_FLAG));

  /* Check the status of the specified SPI flag */
  if ((SPIx->SR1 & SPI_FLAG) != (u16)RESET)
  {
    /* SPI_I2S_FLAG is set */
    bitstatus = SET;
  }
  else
  {
    /* SPI_I2S_FLAG is reset */
    bitstatus = RESET;
  }
  /* Return the SPI_I2S_FLAG status */
  return  bitstatus;
}

/**
  * @brief  Clears the SPIx flag.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
            @arg  SPI0
            @arg  SPI1
  * @param  SPI_FLAG: specifies the SPI flag to clear.         
            @arg  SPI_FLAG_RFOVF
            @arg  SPI_FLAG_RFNETO
  * @retval None
  */
void SPI_ClearFlag(SPI_TypeDef* SPIx, u16 SPI_FLAG)
{
  /* Check the parameters */
  assert_param(IS_SPI_ALL_PERIPH(SPIx));
  assert_param(IS_SPI_CLEAR_FLAG(SPI_FLAG));

  /* Clear the selected SPI flag */
  SPIx->IC |= SPI_FLAG;
}

/**
  * @}
  */

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
