/**
  ******************************************************************************
  * @file    PT32x031x_tim.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the TIM peripheral:
  *            + TimeBase management
  *            + Output Compare management
  *            + Input Capture management
  *            + Interrupts, flags management
  *            + Clocks management  
  *    
  ******************************************************************************
  * @attention
  * 
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_tim.h"


/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup TIM 
  * @brief TIM driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup TIM_Private_Functions
  * @{
  */

/** @defgroup TIM_Group1 TimeBase management functions
 *  @brief   TimeBase management functions 
 *
@verbatim
 ===============================================================================
                 ##### TimeBase management functions #####
 ===============================================================================
  
        *** TIM Driver: how to use it in Timing(Time base) Mode ***
 ===============================================================================
    [..] To use the Timer in Timing(Time base) mode, the following steps are 
         mandatory:
         (#) Fill the TIM_TimeBaseInitStruct with the desired parameters.
         (#) Call TIM_TimeBaseInit(TIMx, &TIM_TimeBaseInitStruct) to configure 
             the Time Base unit with the corresponding configuration.
         (#) Enable the NVIC if you need to generate the update interrupt. 
         (#) Enable the corresponding interrupt using the function 
             TIM_ITConfig(TIMx, TIM_IT_Update). 
         (#) Call the TIM_Cmd(ENABLE) function to enable the TIM counter.
    [..]
        (@) All other functions can be used seperatly to modify, if needed,
            a specific feature of the Timer. 

@endverbatim
  * @{
  */



/**
  * @brief  Initializes the TIMx Time Base Unit peripheral according to 
  *         the specified parameters in the TIM_TimeBaseInitStruct.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral
	          @arg TIM2
            @arg TIM3
            @arg TIM4
  *         @note TIM4 can only TIM_CNT_MODE_UP mode
  * @param  TimeBaseInit: pointer to a TIM_TimeBaseInitTypeDef
  *         structure that contains the configuration information for
  *         the specified TIM peripheral.
	          @param  TIM_AutoReloadValueTimer matching value.
						        This parameter is between 0 and 0xFFFF.
						@param  TIM_PrescalerTimer prescaler coefficient.
						        This parameter is between 0 and 0xFF.
						@param  CounterModeCount direction control bits.
						        @arg  TIM_CNT_MODE_UP
										@arg  IM_CNT_MODE_DOWN
						@param  ResetCounterCount reset control bit when CR2 overflows.
						        @arg  ENABLE
									  @arg  DISABLE
  * @retval None
  */		
void TIM_TimeBaseInit(TIM_TypeDef* TIMx, TIM_TimeBaseInitTypeDef* TimeBaseInit)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_TIM_AutoReloadValue(TimeBaseInit->TIM_AutoReloadValue));  
  assert_param(IS_TIM_Prescaler(TimeBaseInit->TIM_Prescaler));
	assert_param(IS_FUNCTIONAL_STATE(TimeBaseInit->TIM_ResetCounter));
	assert_param(IS_TIM_Direction(TimeBaseInit->TIM_Direction));

  /* Set the counter match value */
  TIMx->ARR = TimeBaseInit->TIM_AutoReloadValue;
	
  /* Set the Prescaler value */
  TIMx->PSC = TimeBaseInit->TIM_Prescaler;

  /* Select the Counter Mode */
	TIMx->CR2 &= ~TIM_CR2_DIR;
	TIMx->CR2 |=  (TimeBaseInit->TIM_Direction);	
	if((TimeBaseInit->TIM_ResetCounter)==ENABLE)
	{
	 TIMx->CR2 |= TIM_CR2_RSTC;
	}
	else
	{
	 TIMx->CR2 &= ~TIM_CR2_RSTC;	
	}	
}


/**
  * @brief  Fills each TIM_TimeBaseInitStruct member with its default value.
  * @param  TimeBaseInit: pointer to a TIM_TimeBaseInitTypeDef structure which will be initialized.
	          @param  TIM_AutoReloadValueTimer matching value.
						        This parameter is between 0 and 0xFFFF.
						@param  TIM_PrescalerTimer prescaler coefficient.
						        This parameter is between 0 and 0xFF.
						@param  CounterModeCount direction control bits.
						        @arg  TIM_CNT_MODE_UP
										@arg  IM_CNT_MODE_DOWN
						@param  ResetCounterCount reset control bit when CR2 overflows.
						        @arg  ENABLE
									  @arg  DISABLE           
  * @retval None
  */
void TIM_TimeBaseStructInit(TIM_TimeBaseInitTypeDef* TimeBaseInit)
{
  /* Set the default configuration */
  TimeBaseInit->TIM_AutoReloadValue = 0xFFFF;
  TimeBaseInit->TIM_Prescaler = 0x0000;
	TimeBaseInit->TIM_Direction=TIM_Direction_Up;
	TimeBaseInit->TIM_ResetCounter = ENABLE;
}


/**
  * @brief  Configures the TIMx TIM_Prescaler.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral.  
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @param  TIM_Prescaler: specifies the Prescaler Register value (0~255)
            This parameter is between 0 and 0xFF.
  * @note   The Prescaler is loaded at the CNTTT update event or immediatly
  * @retval None
  */
void TIM_PrescalerConfig(TIM_TypeDef* TIMx, u8 Prescaler)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_TIM_Prescaler(Prescaler));
  
  /* Set the Prescaler value */
  TIMx->PSC = Prescaler;
}


/**
  * @brief  Configures the TIMx Match Value.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @param  Reload: specifies the Counter reload value (0~65535)
              This parameter is between 0 and 0xFFFF.
  * @note   The value is loaded at the update event or immediatly
  * @retval None
  */
void TIM_SetCounterMatch(TIM_TypeDef* TIMx, u16 Reload)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));	
  assert_param(IS_TIM_AutoReloadValue(Reload));  

  /* Set the counter reload value */
  TIMx->ARR = Reload;
}

/**
  * @brief  Gets the TIMx Counter value at present.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral     
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @retval Counter Register value.
  */
u16 TIM_GetCounter(TIM_TypeDef* TIMx)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  
  /* Get the Counter Register value */
  return  TIMx->CNT;
}

/**
  * @brief  Gets the TIMx Prescaler value.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral     
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @retval Prescaler Register value.
  */
u16 TIM_GetPrescaler(TIM_TypeDef* TIMx)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  
  /* Get the Prescaler Register value */
  return  TIMx->PSC;
}

/**
  * @brief  Specifies the TIMx Counter Mode to be used.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral 
             @arg TIM2
             @arg TIM3
             @arg TIM4
  * @note   TIM4 is not allowed to set counter mode
  * @param  TIM_CenterAlignedMode: specifies the Counter Mode to be used.
  *          This parameter can be one of the following values:
  *            @arg TIM_CNT_MODE_UP: TIM Up Counting Mode
  *            @arg TIM_CNT_MODE_DOWN: TIM Down Counting Mode
  * @retval None
  */
void TIM_DirectionConfig(TIM_TypeDef* TIMx, u8 Direction)
{ 
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_TIM_Direction(Direction));	
  
  /* Select the Counter Mode */
	TIMx->CR2  &= ~TIM_CR2_DIR;
	TIMx->CR2  |=  Direction;
	
}

/**
  * @brief  Enable or Disable update settiongs of the TIMx.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral 
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @param  NewState: This parameter can be ENABLE or DISABLE.
             @arg ENABLE
             @arg DISABLE
  * @retval None
  */
void TIM_UpdateCmd(TIM_TypeDef* TIMx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	

  if (NewState == ENABLE)
  {
    /* Set the update bit */
    TIMx->CR1 |= TIM_CR1_UG;
  }
  else
  {
    /* Reset the update bit */
    TIMx->CR1 &= (~TIM_CR1_UG);
  }
}

/**
  * @brief  Enable or Disable TIMx Counter.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral  
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @param  NewState: This parameter can be ENABLE or DISABLE.
             @arg ENABLE
             @arg DISABLE
  * @retval None
  */
void TIM_Cmd(TIM_TypeDef* TIMx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));	

  if (NewState == ENABLE)
  {
    /* Set the update bit */
    TIMx->CR1 |= TIM_CR1_EN;
  }
  else
  {
    /* Reset the update bit */
    TIMx->CR1 &= (~TIM_CR1_EN);
  }
}

/**
  * @brief  Enables or disables the specified TIMx interrupts.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral 
             @arg TIM2
             @arg TIM3
             @arg TIM4
    @note TIM4 can only TIM_CNT_MODE_UP mode
  * @param  TIM_IT: Specify the TIM interrupts sources to be enabled or disabled.
  *          This parameter can be one of the following values:
  *          @arg TIM_OCIT_ARI : TIM ARR value match interrupt
  * @param  NewState: This parameter can be ENABLE or DISABLE.\
             @arg ENABLE
             @arg DISABLE
  * @retval None
  */
void TIM_ITConfig(TIM_TypeDef* TIMx, u32 TIM_IT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_TIM_IT(TIM_IT));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));
	

    if (NewState != DISABLE)
    {
      /* Enable the interrupt sources */
      TIMx->CR2 |= TIM_IT;
    }
    else
    {
      /* Disable the interrupt sources */
      TIMx->CR2 &= ~TIM_IT;
	
	  }
}

/**
  * @brief  Check the interrupt flag bit for register overflow.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral 
            @arg TIM2
            @arg TIM3
            @arg TIM4
    @note   TIM4 can only TIM_CNT_MODE_UP mode.
  * @param  TIM_FLAG: Specify the flag to be checked.
  *         This parameter can be one of the following values:
  *         @arg TIM_FLAG_MR0       : TIM ARR value match flag
  * @return FlagStatus of TIM_FLAG (SET or RESET).
  */
FlagStatus TIM_GetFlagStatus(TIM_TypeDef* TIMx, u32 TIM_FLAG)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_TIM_FLAG(TIM_FLAG));	

  if ((TIMx->SR & TIM_FLAG) != 0)
  {
    return SET;
  }
  else
  {
    return RESET;
  }
}

/**
  * @brief  Clear the interrupt flag bit of register overflow.
  * @param  TIMx: where x can be 2, 3 and 4 to select the TIM peripheral 
            @arg TIM2
            @arg TIM3
            @arg TIM4
    @note   TIM4 can only TIM_CNT_MODE_UP mode
  * @param  TIM_FLAG: Specify the flag to be cleared.
  *         This parameter can be one of the following values:
  *         @arg TIM_FLAG_MR0: TIM ARR value match flag
  * @retval None
  */
void TIM_ClearFlag(TIM_TypeDef* TIMx, u32 TIM_FLAG)
{
  /* Check the parameters */
  assert_param(IS_TIM_ALL_PERIPH(TIMx));
  assert_param(IS_TIM_FLAG(TIM_FLAG));	
	
  /* Clear the flags */
  TIMx->SR = TIM_FLAG;
}

/**
  * @}
  */

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
