/**
  ******************************************************************************
  * @file    PT32x031x_uart.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Universal synchronous asynchronous receiver
  *          transmitter (UART):
  *           + Initialization and Configuration
  *           + STOP Mode
  *           + Data transfers
  *           + Multi-Processor Communication
  *           + Half-duplex mode
  *           + Smartcard mode
  *           + IrDA mode
  *           + RS485 mode  
  *           + Interrupts and flags management
  *             
  ******************************************************************************
  * @attention
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_uart.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup UART 
  * @brief UART driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup UART_Private_Functions
  * @{
  */

/** @defgroup UART_Group1 Initialization and Configuration functions
 *  @brief   Initialization and Configuration functions 
 *
@verbatim   
 ===============================================================================
          ##### Initialization and Configuration functions #####
 ===============================================================================
    [..]
        This subsection provides a set of functions allowing to initialize the UART 
        in asynchronous and in synchronous modes.
        (+) For the asynchronous mode only these parameters can be configured: 
          (++) Baud Rate.
          (++) Word Length.
          (++) Stop Bit.
          (++) Parity: If the parity is enabled, then the MSB bit of the data written
               in the data register is transmitted but is changed by the parity bit.
               Depending on the frame length defined by the MODE[2:0] bit,
               the possible UART frame formats are as listed in the following table:

   +------------------------------------------------------------------------+     
   |   M2 bit |   M1 bit|  M0 bit   |            UART frame                 |
   |--------- |---------------------|---------------------------------------|             
   |    0     |    0    |    0      |    | SB | 8 bit data | STB |          |
   |--------- |---------|-----------|---------------------------------------|  
   |    0     |    1    |    1      |    | SB | 7 bit data | PB | STB |     |
   |--------- |---------|-----------|---------------------------------------|  
   |    1     |    0    |    0      |    | SB | 9 bit data | STB |          |
   |--------- |---------|-----------|---------------------------------------|  
   |    1     |    1    |    1      |    | SB | 8 bit data | PB | STB |     |
   +------------------------------------------------------------------------+            

          (++) Receiver/transmitter modes.
    [..] The UART_Init() function follows the UART  asynchronous configuration 
         procedure(details for the procedure are available in reference manual.

@endverbatim
  * @{
  */
/**
  * @brief  Initializes the UARTx peripheral according to the specified
  *         parameters in the UART_InitStruct .
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral. 
            @arg UART0
						@arg UART1
  * @param  UART_InitStruct: pointer to a UART_InitTypeDef structure that contains
  *         the configuration information for the specified UART peripheral.
	          @param  UART_BaudRateBaud rate of UART communication
                  	This parameter must be a number between 0 and 0x5B8D81.
            @param  UART_FrameLengthUART Frame length
                   @arg  UARTM_8D		
                   @arg  UARTM_7D_P
                   @arg  UARTM_9D
                   @arg	 UARTM_8D_W	
                   @arg	 UARTM_8D_P		
						@param  UART_StopBit: Stop bit length.
                   @arg	 UART05StopBits	
                   @arg  UART1StopBit
                   @arg  UART15StopBits
                   @arg	 UART2StopBits
						@param  UART_SampleRate: Serial communication symbol sampling rate
                   @arg  UART_SAMPLERATEX4
                   @arg  UART_SAMPLERATEX8
                   @arg	 UART_SAMPLERATEX16
					  @param  UART_RXEN: Receive enable and disable.
                   @arg  Enable
                   @arg	 Disable								 
  * @retval None
  */
void UART_Init(UART_TypeDef* UARTx, UART_InitTypeDef* UART_InitStruct)
{
  u32  tmpreg = 0 ,divider = 0,pclk = 0;

  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_BAUDRATE(UART_InitStruct->UART_BaudRate));  
  assert_param(IS_UART_WORD_LENGTH(UART_InitStruct->UART_WordLength));
  assert_param(IS_UART_STOPBITS(UART_InitStruct->UART_StopBit));
	assert_param(IS_UART_RXSTATUS(UART_InitStruct->UART_RXEN));
	
  UARTx->RXFR = UART_BRR_BR;
  UARTx->TXFR = UART_TXFR_TXFR;
	
	tmpreg &= ~(UART_CR_WP | UART_CR_STOP | UART_CR_PS);
	tmpreg |= (UART_InitStruct -> UART_WordLength | UART_InitStruct -> UART_StopBit | UART_InitStruct -> UART_Parity);  
	UARTx->CR = tmpreg; 
  if (UART_InitStruct -> UART_RXEN != DISABLE)
	{		
    UARTx->CR |=UART_CR_RE;
	}		
  else
	{		
    UARTx->CR &=~UART_CR_RE;
	}
	 pclk=RCC_GetClockFreq(RCC_PCLK);
   tmpreg  =  (pclk) % (16 * UART_InitStruct->UART_BaudRate);
   divider =  (pclk) / (16 * UART_InitStruct->UART_BaudRate);
	if(divider==0)
	{
	 UARTx->BRR=1;
	}
	else
	{
		if (tmpreg > ((16/2) * UART_InitStruct->UART_BaudRate))
		{
			UARTx->BRR = (u16) (divider + 1);  	
		}
		else
		{
			UARTx->BRR = (u16) divider; 	
		} 	
	}
} 


/**
  * @brief  Fills each UART_InitStruct member with its default value.
  * @param  UART_InitStruct: pointer to a UART_InitTypeDef structure
  *         which will be initialized.
            @param  UART_BaudRateBaud rate of UART communication
                  	This parameter must be a number between 0 and 0x5B8D81.
            @param  UART_FrameLengthUART Frame length
                   @arg  UARTM_8D		
                   @arg  UARTM_7D_P
                   @arg  UARTM_9D
                   @arg	 UARTM_8D_W	
                   @arg	 UARTM_8D_P		
						@param  UART_StopBit: Stop bit length.
                   @arg	 UART05StopBits	
                   @arg  UART1StopBit
                   @arg  UART15StopBits
                   @arg	 UART2StopBits
						@param  UART_SampleRate: Serial communication symbol sampling rate
                   @arg  UART_SAMPLERATEX4
                   @arg  UART_SAMPLERATEX8
                   @arg	 UART_SAMPLERATEX16
					  @param  UART_RXEN: Receive enable and disable.
                   @arg  Enable
                   @arg	 Disable	
						@param  UART_Parity: Parity form.						
  * @retval None
  */
void UART_StructInit(UART_InitTypeDef* UART_InitStruct)
{
  /* USART_InitStruct members default value */
  UART_InitStruct->UART_BaudRate = 19200;
  UART_InitStruct->UART_WordLength = UARTM_8D_P;
  UART_InitStruct->UART_StopBit = UART1StopBit;
  UART_InitStruct->UART_Parity = UART_ODD_PARITY;
  UART_InitStruct->UART_RXEN = ENABLE;
}

/**
  * @brief  Enables or disables the specified UART peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral. 
            @arg UART0
						@arg UART1
  * @param  NewState: new state of the USARTx peripheral.
  *         This parameter can be: ENABLE or DISABLE.
             @arg  Enable
             @arg	 Disable
  * @retval None
  */
void UART_Cmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if (NewState != DISABLE)
  {
    /* Enable the selected UART by setting the BRUN bit in the CR register */
    UARTx->CR |= UART_CR_EN;
  }
  else
  {
    /* Disable the selected UART by clearing the BRUN bit in the CR register */
    UARTx->CR &= (u32)~((u32)UART_CR_EN);
  }
}

/**
  * @brief  Enables or disables the UART's  receiver.
    @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  NewState: new state of the UART transfer direction.
  *         This parameter can be: ENABLE or DISABLE.
            @arg  Enable
            @arg	 Disable
  * @retval None
  */
void UART_RxConfig(UART_TypeDef* UARTx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_FUNCTIONAL_STATE(NewState)); 

  if (NewState != DISABLE)
  {
    /* Enable the UART's transfer interface as Receiver by setting the RXEN bits 
       in the UART CR register */
    UARTx->CR |= UART_CR_RE;
  }
  else
  {
    /* Disable the UART's transfer interface as Receiver by setting the RXEN bits
       in the UART CR register */
    UARTx->CR &= (u32)~UART_CR_RE;
  }
}

/**
  * @brief  Enables or disables the Pin(s) active level inversion.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  UART_InvPin: specifies the UART pin(s) to invert.
  *          This parameter can be any combination of the following values:
  *            @arg UART_InvPin_Tx: UART Tx pin active level inversion.
  *            @arg UART_InvPin_Rx: UART Rx pin active level inversion.
  * @param  NewState: new active level status for the UART pin(s).
  *          This parameter can be:
  *            @arg ENABLE: pin(s) signal values are inverted (Vdd =0, Gnd =1).
  *            @arg DISABLE: pin(s) signal works using the standard logic levels (Vdd =1, Gnd =0).
  * @note   This function has to be called before calling UART_Cmd() function.  
  * @retval None
  */

void UART_InverPolarityCmd(UART_TypeDef* UARTx, u32 UART_InvPin, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_INVERSTION_PIN(UART_InvPin));  
  assert_param(IS_FUNCTIONAL_STATE(NewState)); 

  if (NewState != DISABLE)
  {
    UARTx->CR |= UART_InvPin;
  }
  else
  {
    UARTx->CR &= (u32)~UART_InvPin;
  }
}

/**
  * @brief  Sets the receiver Time Out value.
  * @param  UARTx: where x can be 0 or 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  UART_ReceiverTimeOut: specifies the Receiver Time Out value.\
            This parameter must be a number between 0 and 0xFF.
  * @retval None
  */
void UART_SetReceiverTimeOut(UART_TypeDef* UARTx, u32 UART_ReceiverTimeOut)
{    
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_TIMEOUT(UART_ReceiverTimeOut));

  /* Clear the receiver Time Out value by clearing the RTO[7:0] bits in the RTOR
     register  */
  UARTx->TO &= (u32)~((u32)UART_TO_TO);
  /* Set the receiver Time Out value by setting the RTO[7:0] bits in the RTOR
     register  */
  UARTx->TO |= UART_ReceiverTimeOut;
}

/**
  * @brief  Transmits single data through the UARTx peripheral.
  * @param  UARTx: where x can be from 0 or 1 to select the UART peripheral.
	          @arg UART0
						@arg UART1
  * @param  Data: the data to transmit.
  * @retval None
  */
void UART_SendData(UART_TypeDef* UARTx, u16 Data)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_DATA(Data)); 
  /* While the TxFIFO contain 8 characters. */
  while((UARTx->SR & UART_SR_TXF)); 
  /* Transmit Data */
  UARTx->DR = (Data & (u16)0x01FF);
}

/**
  * @brief  Transmits single string through the UARTx peripheral.
  * @param  UARTx: where x can be from 0 or 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  String: Pointer to the string to transmit.
  * @retval None
  */
void UART_StringSend(UART_TypeDef* UARTx, u8 *String)
{
  while(*String != '\0')
	{	
    while((UARTx->SR & UART_SR_TXF)); 
    UART_SendData(UARTx, *String);
		String++;	
	}
  UART_SendData(UARTx, 0x00);  
}

/**
  * @brief  Returns the most recent received data by the UARTx peripheral.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @retval The received data.
  */
u16 UART_ReceiveData(UART_TypeDef* UARTx)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx)); 
  /* Receive Data */
  return (u16)(UARTx->DR & (u16)0x01FF);
}

/**
  * @brief  Whether the receiver has timed out when receiving string.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @Data   Data: Pointer to the receivde data.
  * @param  UART_ReceiverTimeOut: The time out period.
  * @retval Is there a timeout.
  */
u16 UART_ReceiveData_Timeout(UART_TypeDef* UARTx, u16 *Data, u32 UART_ReceiverTimeOut)
{
  u16 wStatus;
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx)); 
  /* Reload the Timeout counter */	
  UART_SetReceiverTimeOut(UARTx, UART_ReceiverTimeOut);
  /* while the RxFIFO is empty and no Timeoutidle */	
	while (!((wStatus=UARTx->SR) & (UART_SR_IDLE|UART_SR_RXHF|UART_SR_RXNE)));
  /* Receive Data */
	*Data = (u16)(UARTx->DR & (u16)0x01FF); 
  return (wStatus);
}

/**
  * @brief  Returns the most recent received string by the UARTx peripheral
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @Data   Data: Pointer to the reveive data.
  * @retval The received data.
  */
u16 UART_StringReceive(UART_TypeDef* UARTx, u8 *Data)
{
  u8 *pSTRING;
  u16 wStatus;
  pSTRING=Data;
  do
  {
    /* while the RxFIFO is empty and no Timeoutidle */		
    while (!((wStatus=UARTx->SR) & (UART_SR_RXHF|UART_SR_RXNE)));// 
    /* Read the Receive Buffer */			
    *(pSTRING++) = (u8)UARTx->DR; 
  }
	
	while((*(pSTRING - 1)!=0x0D)&(*(pSTRING - 1)!='\0'));
  *(pSTRING - 1)='\0';
	
  return wStatus;
}

/**
  * @brief  This function the loop back mode of the selected UART.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  NewState: new active level status for the UART pin(s).
  *         This parameter can be:
  *         @arg ENABLE: Loop back mode .
  *         @arg DISABLE: Normal communication mode .
  * @retval None
  */
void UART_LoopBackConfig(UART_TypeDef* UARTx, FunctionalState NewStatus)
{
	
	if (NewStatus != DISABLE)
  {
    UARTx->CR |= UART_CR_LPB;
  }
  else
  {
    UARTx->CR &= ~UART_CR_LPB;
  }

}

/**
  * @brief  This function reset the Rx and the Tx FIFOs
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  FIFO: Transmit FIFO or receive FIFO for the UART .
  *         This parameter can be:
  *         @arg Rx_FIFO: Receive FIFO .
  *         @arg Tx_FIFO: Transmit FIFO .
  * @retval None.
  */
void UART_FifoReset(UART_TypeDef * UARTx,  UART_FifoTypeDef FIFO)
{

  switch (FIFO)
  {
       case Rx_FIFO : UARTx->RXFR=UART_RXFR_RXFR;break;
       case Tx_FIFO : UARTx->TXFR=UART_TXFR_TXFR;break;
  }
}

/**
  * @brief  Enables or disables the UART's IrDA interface.
  * @param  UARTx: where x can be 0 to select the UART peripheral.
	          @arg UART0
						@arg UART1
  * @param  NewState: new state of the IrDA mode.
  *          This parameter can be: ENABLE or DISABLE.
	          @arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void UART_IrDACmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_USART_0_PERIPH(UARTx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the IrDA mode by setting the EN bit in the IRR register */
    UARTx->IMCR |= UART_IMCR_IRE;
  }
  else
  {
    /* Disable the IrDA mode by clearing the EN bit in the IRR register */
    UARTx->IMCR &= (u32)~((u32)UART_IMCR_IRE);
  }
}

/**
  * @brief  Configures the IrDA's Pin polarity
  * @param  UARTx: where x can be 0 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  NewState: new defined levels for the USART data.
  *          This parameter can be:
  *         @arg ENABLE: pin(s) signal values are inverted (Vdd =0, Gnd =1).                       
  *         @arg DISABLEpin(s) signal works using the standard logic levels (Vdd =1, Gnd =0).

  * @note   This function has to be called before calling UART_Cmd() function.  
  * @retval None
  */
void UART_IrDAInvPinCmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_USART_0_PERIPH(UARTx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the binary data inversion feature by setting the DATAINV bit in 
       the CR2 register */
    UARTx->IMCR |= UART_IMCR_IRE;
  }
  else
  {
    /* Disable the binary data inversion feature by clearing the DATAINV bit in 
       the CR2 register */
    UARTx->IMCR &= (u32)~((u32)UART_IMCR_IRE);
  }
}

/**
  * @brief  Duty cycle control of infrared transmission.
  * @param  UARTx: where x can be 0,1 to select the USART peripheral.
            @arg UART0
						@arg UART1
  * @param  UART_DutyCycle: Infrared duty cycle.
            This parameter is between 0 and 0x64.
  * @retval None
  */
void UART_SetDutyCycle(UART_TypeDef* UARTx, u8 UART_DutyCycle)
{ 
  /* Check the parameters */
  assert_param(IS_USART_0_PERIPH(UARTx));
  assert_param(IS_UART_DUTYCYCLE(UART_DutyCycle));
  
  /* Clear the IrDA's modulation PWM duty cycle */
  UARTx->IMDCR &= ~UART_IMDCR_DUTY;
  /* Set the IrDA's modulation PWM duty cycle*/
  UARTx->IMDCR |= (u32)((u32)UART_DutyCycle);
}

/**
  * @brief  Enables or disables the UART's Half Duplex communication.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.   
	          @arg UART0
						@arg UART1
  * @param  NewState: new state of the UART Communication.
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_HalfDuplexCmd(UART_TypeDef* UARTx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if (NewState != DISABLE)
  {
    /* Enable the Half-Duplex mode by setting the OWE bit in the CR register */
    UARTx->CR |= UART_CR_SLME;
  }
  else
  {
    /* Disable the Half-Duplex mode by clearing the OWE bit in the CR register */
    UARTx->CR  &= (u32)~((u32)UART_CR_SLME);
  }
}

/**
  * @brief  Enables or disables the UART's transmitter or receiver.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
            @arg UART0
						@arg UART1
  * @param  UART_Direction: specifies the UART direction.
  *          This parameter can be any combination of the following values:
  *         @arg UART_Mode_Tx: USART Transmitter
  *         @arg UART_Mode_Rx: USART Receiver
  * @param  NewState: new state of the USART transfer direction.
  *         This parameter can be: ENABLE or DISABLE.  
  * @retval None
  */
void UART_HalfDuplex_DirectionModeCmd(UART_TypeDef* UARTx, u32 UART_DirectionMode)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_MODE(UART_DirectionMode));

  switch (UART_DirectionMode)
  {
    /* Enable the UART's transfer interface as Transmitter by setting the OWD bits 
       in the UART CR register */		
    case UART_Mode_Tx : 
			UARTx->CR |= UART_CR_SLDS;
		  break;
    /* Disable the UART's transfer interface as Receiver by clearing OWD bits
       in the UART CR register */		
    case UART_Mode_Rx : 
			UARTx->CR &= (u32)~UART_CR_SLDS;
		  break;
    default : 
			break;
  }	
}

/**
  * @brief  Enables or disables the specified UART interrupts.
  * @param  UARTx: where x can be from 1 to 8 to select the UART peripheral.
  * @param  UART_IT: specifies the UART interrupt sources to be enabled or disabled.
  * @param  NewState: new state of the specified USARTx interrupts.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void UART_ITConfig(UART_TypeDef* UARTx, u32 UART_IT, FunctionalState NewState)
{

  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_CONFIG_IT(UART_IT));
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  
  if (NewState != DISABLE)
  {
    UARTx->IE |= UART_IT;
  }
  else
  {
    UARTx->IE &= ~UART_IT;
  }
}


/**
  * @brief  Checks whether the specified UART flag is set or not.
  * @param  UARTx: where x can be from 0 to 1 to select the UART peripheral.
  * @param  UART_FLAG: specifies the flag to check.
  * @retval The new state of UART_FLAG (SET or RESET).
  */
FlagStatus UART_GetFlagStatus(UART_TypeDef* UARTx, u32 UART_FLAG)
{
  FlagStatus bitstatus = RESET;
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_FLAG(UART_FLAG));
  
  if ((UARTx->SR & UART_FLAG) != (u16)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  return bitstatus;
}

/**
  * @brief  Clears the UARTx's pending flags.
  * @param  UARTx: where x can be from 0 to 1 to select the USART peripheral. 
            @arg UART0
						@arg UART1
  * @param  UART_FLAG: specifies the flag to clear.
  *         This parameter can be any combination of the following values:
  *         @arg UART_CLEAR_FLAG_PERR:  Parity error.
  *         @arg UART_CLEAR_FLAG_FERR:  Frame error.
  *         @arg UART_CLEAR_FLAG_OVERR: Receiver FIFO buffer overflow.
  * @retval None
  */
void UART_ClearFlag(UART_TypeDef* UARTx, u32 UART_CLEAR_FLAG)
{
  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_CLEAR_FLAG(UART_CLEAR_FLAG));
     
  UARTx->SR = UART_CLEAR_FLAG;
}

/**
  * @brief  Checks whether the UATR interrupt has occurred or not.
  * @param  UARTx: where x can be from 0 to 1 to select the USART peripheral. 
            @arg UART0
						@arg UART1
  * @param  UART_IT: specifies the UART interrupt source to check. 
  * @retval The new state of UART_IT (SET or RESET).
  */

ITStatus UART_GetITStatus(UART_TypeDef* UARTx,u32 UART_IT)
{
  ITStatus bitstatus = RESET;
  u32 enablestatus = 0;

  /* Check the parameters */
  assert_param(IS_UART_ALL_PERIPH(UARTx));
  assert_param(IS_UART_GET_IT(UART_IT));

  /* Get the UART_IT enable bit status */
  enablestatus = (u32)(UARTx->IE & UART_IT); 

  /* Check the status of the specified ADC interrupt */
  if (((u32)(UARTx->SR & UART_IT) != (u32)RESET) && (enablestatus != (u32)RESET))
  {
    /* UART_IT is set */
    bitstatus = SET;
  }
  else
  {
    /* UART_IT is reset */
    bitstatus = RESET;
  }
  /* Return the UART_IT status */
  return  bitstatus;
}

/**
  * @}
  */
/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
