/**
  ******************************************************************************
  * @file    PT32x031x_wdg.c
  * @author  Application Team
  * @version V1.0.0
  * @date    2023/12/1   
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Independent watchdog (IWDG) peripheral:           
  *           + Prescaler and Counter configuration
  *           + IWDG activation
  *           + Flag management
  *
  *    
  ******************************************************************************
  * @attention
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "PT32x031x_wdg.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup IWDG 
  * @brief IWDG driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* ---------------------- IWDG registers bit mask ----------------------------*/
/* KR register bit mask */
#define KR_KEY_DISABLE   ((u32)0xFFFFFFFF)
#define KR_KEY_ENABLE    ((u32)0x1ACCE551)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup IWDG_Private_Functions
  * @{
  */

/** @defgroup IWDG_Group1 Prescaler and Counter configuration functions
 *  @brief   Prescaler and Counter configuration functions
 *
@verbatim   
  ==============================================================================
            ##### Prescaler and Counter configuration functions #####
  ==============================================================================  

@endverbatim
  * @{
  */

/**
  * @brief  Enables or disables write access all registers.
	* @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @param  IWDG_WriteAccess: new state of write access to all registers.
  *         This parameter can be one of the following values:
  *         @arg WDG_LockKey_Unlock
  *         @arg WDG_LockKey_Lock
  * @retval None
  */
void WDG_LockCmd(WDG_TypeDef* WDGx, uint32_t WDG_WriteAccess)
{
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
  assert_param(IS_WDG_WRITE_ACCESS(WDG_WriteAccess));
  WDGx->LOCK = WDG_WriteAccess;
}


/**
  * @brief  Sets WDG Reload value.
  * @param  WDGx: specifies watchdog type.
  * @param  Reload: specifies the WDG Reload value.
  *         This parameter must be a number between 0 and 0xFFFFFFFF.
            @arg  IWDG
            @arg  SWDG
  * @retval None
  */
void WDG_SetReload(WDG_TypeDef* WDGx, uint32_t Reload)
{
  /* Check the parameters */
  assert_param(IS_WDG_RELOAD(Reload));
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
  WDGx->RLR = Reload;
}

/**
  * @brief  Reloads WDG counter with value defined in the reload register
  *   (write access to all registers disabled).
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @retval None
  */
void WDG_ReloadCounter(WDG_TypeDef* WDGx)
{
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));	
  WDGx->KR = 0xFFFFFFFF;
}



/**
  * @}
  */

/** @defgroup IWDG_Group2 IWDG activation function
 *  @brief   IWDG activation function 
 *
@verbatim   
 ==============================================================================
                          ##### IWDG activation function #####
 ==============================================================================  

@endverbatim
  * @{
  */

/**
  * @brief  Enables WDG (write access to all registers disabled).
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @param  NewState: This parameter can be ENABLE or DISABLE.
          	@arg  ENABLE
            @arg  DISABLE
  * @retval None
  */
void WDG_AccessEnable(WDG_TypeDef* WDGx, FunctionalState NewState)
{
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));		
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if (NewState != DISABLE)
  {
    /* Enable the WDG */
    WDGx->LOCK = KR_KEY_ENABLE;
  }
  else
  {
    /* Disable the WDG */
    WDGx->LOCK = KR_KEY_DISABLE;
  }
}

/**
  * @brief  Check the locking status of WDG
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @retval Return the locking status of WDG 
  */
FlagStatus WDG_GetLockStatus(WDG_TypeDef* WDGx)
{
  FlagStatus bitstatus = RESET;
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));		
  if ((WDGx->LOCK & WDG_KR_LOCK) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  /* Return the flag status */
  return bitstatus;
}


/**
  * @brief  Enables or disables the specified WDG
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @param  NewState: new state of the WDG. 
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void WDG_Cmd(WDG_TypeDef* WDGx, FunctionalState NewState)
{
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
  if (NewState != DISABLE)
  {
    /* Enable the IWDG */
    WDGx->CR |= WDG_CR_INTE;
  }
  else
  {
    /* Disable the IWDG */
    WDGx->CR &= ~WDG_CR_INTE;
  }
}
/**
  * @brief  Enables or disables the WDG RESET.
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @param  NewState: new state of the WDG. 
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void WDG_ResetCmd(WDG_TypeDef* WDGx, FunctionalState NewState)
{
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
  if (NewState != DISABLE)
  {
    /* Enable the IWDG RESET */
    WDGx->CR |= WDG_CR_RSTE;
  }
  else
  {
    /* Disable the IWDG RESET*/
    WDGx->CR &= ~WDG_CR_RSTE;
  }
}

/**
  * @brief  WDG debug pending control
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG
  * @param  NewState: 
  *         @arg  ENABLE  
            @arg  DISABLE
  * @retval None
  */
void WDG_DbgeRunCmd(WDG_TypeDef* WDGx, FunctionalState NewState)
{
  /* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
  if (NewState != DISABLE)
  {
    /* Enable the IWDG RESET */
    WDGx->CR |= WDG_CR_DBGE;
  }
  else
  {
    /* Disable the IWDG RESET*/
    WDGx->CR &= ~WDG_CR_DBGE;
  }
}
/**
  * @}
  */

/** @defgroup IWDG_Group3 Flag management function 
 *  @brief  Flag management function  
 *
@verbatim   
 ===============================================================================
                      ##### Flag management function ##### 
 ===============================================================================  

@endverbatim
  * @{
  */

/**
  * @brief  Checks whether the specified WDG flag is set or not.
  * @param  WDGx: specifies watchdog type.
            @arg  IWDG
            @arg  SWDG          
  * @param  WDG_FLAGhis parameter can be one of the following values:
  *         @arg  WDG_FLAG_RVU  
  * @retval The new state of WDG_FLAG (SET or RESET).
  */
FlagStatus WDG_GetFlagStatus(WDG_TypeDef* WDGx, u32 WDG_FLAG)
{
  FlagStatus bitstatus = RESET;
  /* Check the parameters */
  assert_param(IS_WDG_FLAG(WDG_FLAG));
	assert_param(IS_WDG_ALL_PERIPH(WDGx));		
  if ((WDGx->SR & WDG_FLAG) != (u32)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  /* Return the flag status */
  return bitstatus;
}



/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
